function holdForTouch(side)
	game:findQueue{
		type = "async",
		context = "entity",
		id = "wait_for_touch",
		side = side,
		update = function(self, dt, speed)
			self.finished = self.parent.physics.touch[self.side]
		end
 	}
end

function createShield(parent, slice)
	local shield = {
		x = parent.x + slice.x,
		y = parent.y + slice.y,
		width = slice.w,
		height = slice.h,
		type = "shield",
		active = true,
		parent = parent
	}

	phys():add(shield, shield.x, shield.y, shield.width, shield.height)
	return shield
end

function drawSlice(entity, slice, x, y)
	local ls = entity.sprite.slice
	local lx = entity.x
	local ly = entity.y
	entity.x = x
	entity.y = y
	entity.sprite.slice = slice
	game:getSystem("sprite"):render(entity)
	entity.x = lx
	entity.y = ly
	entity.sprite.slice = ls
end

function explode(x, y, scale, force)
	game.audio:play("assets/sounds/break.wav", "sfx")
	local lc = game.targetContext
	game.targetContext = "game"
	fx("explode", x, y, true)
	game.targetContext = lc
	force = force or 250

	local exp = {
		x = x - (scale / 2),
		y = y - (scale / 2),
		width = scale,
		height = scale
	}
	--phys():update(self.parent, exp.x, exp.y, exp.width, exp.height)
	local items, len = phys():queryRect(exp.x, exp.y, exp.width, exp.height)

	for index, item in pairs(items) do
		if item.physics and item.physics.collides and (item.behavior and item.behavior:doesExplode()) then
			local speed = math.lerp(force, 0, math.dist(exp.x + (exp.width / 2), exp.y + (exp.height / 2), item.center.x, item.center.y) / scale)
			local vel = math.rotate(speed, speed,
				math.angle(
					exp.x + (exp.width / 2),
					exp.y + (exp.height / 2),
					item.x + item.physics.offset.x + (item.physics.width / 2),
					item.y + item.physics.offset.y + (item.physics.height / 2)
				) - math.rad(45)
			)

			item.physics.velocity.x = vel.x
			item.physics.velocity.y = vel.y

			if item.behavior.on_hit then
				item.behavior:on_hit("explosion")
			end

			if item.class == "player" then
				freeze(0.075)
			end
		end
	end
end

function freeze(time)
	game:findQueue{
		type = "async",
		context = "game",
		id = "freeze",
		time = time,
		age = 0,
		start = function(self)
			self.ls = game.speed
			game.speed = 0.1
		end,
		update = function(self, dt, speed)
			self.age = self.age + dt

			if self.age > self.time then
				self.finished = true
			end
		end,
		finish = function(self)
			game.speed = self.ls
		end
	}
end

function removeSelf()
	game:findQueue{
		type = "instant",
		context = "entity",
		id = "remove_self",
		run = function(self)
			game:remove(self.parent)
		end
	}
end

function toggleDoor(door)
	if not game:get(door) then
		error "attempted to open door that doesn't exist"
	end

	local door = game:get(door)
	if door.behavior.state.current == "open" then
		door.behavior.state:close(door)
	else
		door.behavior.state:open(door)
	end
end

function sprite(type, options)
	return game:add(table.merge({
		class = "sprite",
		queue = {},
		sprite = {
			sprite = f("assets/sprites/%s.png", type)
		},
		animation = {
			schema = f("assets/sprites/%s.json", type)
		}
	}, options or {}))
end

function syncHitbox(entity, slice)
	local hitbox = entity.sprite.slices[slice].frames[entity.animation.index].bounds
	entity.physics.width = hitbox.w
	entity.physics.height = hitbox.h
	entity.physics.offset = {
		x = hitbox.x - (entity.sprite.origin.x * entity.animation.width),
		y = hitbox.y - (entity.sprite.origin.y * entity.animation.height)
	}
end

function waitFor(condition)
	game:findQueue{
		type = "async",
		context = "entity",
		id = "wait_for_touch",
		condition = condition,
		update = function(self, dt, speed)
			self.finished = self:condition()
		end
	}
end
