--
--
--
--
--

return function()
	return {
		hitEnemies = {},
		bounced = false,
		awake = function(self)
			self.parent.physics.collide = function(self, other)
				if other.physics then
					if other.physics.bounces then
						self.behavior.bounced = true
						return "bounce"
					end

					if (other.physics.blocksBall and other.physics.collides) then
						return "slide"
					end
				elseif other.type == "map" then
					print (other.properties.collideType)
					return other.properties.collideType or "slide"
				elseif other.type == "shield" and other.active then
					_print "Hit shield."
					return "bounce"
				else
					return "cross"
				end
			end
		end,
		update = function(self, dt, speed)
			self.bounced = false

			local hb = self.parent.sprite.slices.hitbox.frames[self.parent.animation.index].bounds

			if self.shouldExplode then
				self:explode()
			end

			self.parent.physics.width = hb.w
			self.parent.physics.height = hb.h
			self.parent.physics.offset.x = -(self.parent.sprite.origin.x * self.parent.animation.width) + hb.x
			self.parent.physics.offset.y = -(self.parent.sprite.origin.y * self.parent.animation.height) + hb.y

			self.parent.sprite.angle = math.angle(
				self.parent.x, self.parent.y,
				self.parent.x + (self.parent.physics.velocity.x * (dt * speed)),
				self.parent.y + (self.parent.physics.velocity.y * (dt * speed))
			)

			if self.parent.bind then
				if self.parent.x < 0
				or self.parent.x > game.roomWidth
				or self.parent.y < 0
				or self.parent.y > game.roomHeight then
					print "removing for out of bounds"
					game:remove(self.parent)
				end
			end

			if self.parent.charged then
				self.parent.animation:setAnimation "charged"
			elseif math.abs(self.parent.physics.velocity.x) + math.abs(self.parent.physics.velocity.y) > 500 then
				self.parent.animation:setAnimation "fast"
			else
				self.parent.animation:setAnimation "idle"
			end
		end,
		on_hit = function(self, interactor)

		end,
		isEnemy = function(self)
			return false
		end,
		doesExplode = function(self)
			return false
		end,
		collide = function(self, collision, isOther)
			local other = isOther and collision.item or collision.other

			if (other.physics and (other.physics.blocksBall and other.physics.collides)) or other.type == "map" then
				if collision.type ~= "bounce" then
					self.shouldExplode = true
				end
			end

			printf("collided with object: %s", other.name or other.type)

			if collision.type == "bounce" then
				_print "bounced."

				if other.type == "shield" and other.active then
					_print "YOU HIT THE SHIELD"
				end

				local lc = game.context
				game.context = self.parent
				game.audio:play("assets/sounds/tink.wav", "sfx")
				fx("tink", self.parent.x, self.parent.y, true)
				game.context = lc
			end
		end,
		explode = function(self)
			local lc = game.context
			game.context = self.parent
			game.camera:shake(0.3)
			removeSelf()
			explode(self.parent.x, self.parent.y, 8 * 8, self.parent.charged and 325 or 250)

			game.context = lc
		end
	}
end
