

return function()
	return {
		dir = -1,
		vision = 500,
		pattern = 2,
		walkSpeed = 48,
		awake = function(self)
			if self.parent.direction == "right" then
				self.parent.sprite.flip.x = true
			end

			if self.parent.direction == "up" then
				self.dir = -self.dir
				self.parent.sprite.flip.y = true
				self.parent.sprite.angle = math.rad(90)
			end

			local hitbox = self.parent.sprite.slices.hitbox.frames[self.parent.animation.index]
			local flipX = self.parent.sprite.flip.x
			local flipY = self.parent.sprite.flip.y

			self.parent.physics.width = hitbox.bounds.w
			self.parent.physics.offset.x = flipX and self.parent.animation.width - hitbox.bounds.w - hitbox.bounds.x or hitbox.bounds.x
			self.parent.physics.offset.y = flipY and self.parent.animation.height - hitbox.bounds.h - hitbox.bounds.y or hitbox.bounds.y
			self.parent.physics.height = hitbox.bounds.h

			self.parent.x = self.parent.x - self.parent.physics.offset.x
			self.parent.y = self.parent.y - self.parent.physics.offset.y


			self.state = game:machine{
				initial = "walk",
				events = {
					{name = "kill", from = {"walk"}, to = "dead"}
				},
				callbacks = {
					onkill = function(self, event, to, from, parent)
						local lc = game.context
						game.context = parent
						parent.queue:clear()
						velocity {x = 0, y = 0}
						fx("explode", parent.x + 12, parent.y + 24, true)
						removeSelf()
						game.context = lc
					end
				}
			}

			self.parent.physics._collide = self.parent.physics.collide
			self.parent.physics.collide = function(self, other)
				if other.class == "ball" then
					return "cross"
				elseif other.class == "player" then
					if other.behavior.invulnerable then
						return "cross"
					else
						return "slide"
					end
				else
					return self.physics._collide(self, other)
				end
			end
		end,
		update = function(self, dt, speed)
			local hitbox = self.parent.sprite.slices.hitbox.frames[self.parent.animation.index]
			local flip = self.parent.sprite.flip.x

			self.parent.physics.width = hitbox.bounds.w
			self.parent.physics.offset.x = flip and self.parent.animation.width - hitbox.bounds.w - hitbox.bounds.x or hitbox.bounds.x
			self.parent.physics.offset.y = hitbox.bounds.y
			self.parent.physics.height = hitbox.bounds.h

			self.parent.physics.gravity = {
				x = 0,
				y = 0
			}

			if self.parent.direction == "left" or self.parent.direction == "right" then
				self.parent.physics.velocity.max.x = 0
			else
				self.parent.physics.velocity.max.y = 0
			end



			if self[self.state.current] then
				self[self.state.current](self, dt, speed)
			end
		end,
		isEnemy = function(self)
			return true
		end,
		collide = function(self, collision, isOther)
			local other = isOther and collision.item or collision.other

			if other.class == "ball" then
				local hitEnemies = other.behavior.hitEnemies
				local doHit = true
				for index, enemy in pairs(hitEnemies) do
					if enemy == self.parent then
						doHit = false
					end
				end

				if doHit then
					other.behavior.shouldBounce = true
					self:takeHit()
					table.insert(other.behavior.hitEnemies, self.parent)
					print "HURT"
				end
			end

			if other.class == "player" then
				other.behavior.state:kill(other)
			end
		end,
		idle = function(self, dt, speed)
			self.state:walk(self.parent)
		end,
		walk = function(self, dt, speed)
			local flip = false

			local testX = 0
			local testY = 0

			if self.parent.direction == "down" then
				testX = self.parent.x + (self.parent.physics.offset.x) + (self.parent.physics.width / 2) + (self.parent.physics.velocity.x * (dt * speed))
				testY = self.parent.y + self.parent.physics.height + self.parent.physics.offset.y + 4

				if self.parent.physics.touch.left or self.parent.physics.touch.right then
					flip = true
				end
			elseif self.parent.direction == "up" then
				testX = self.parent.x + (self.parent.physics.offset.x) + (self.parent.physics.width / 2) + (self.parent.physics.velocity.x * (dt * speed))
				testY = self.parent.y - 4

				if self.parent.physics.touch.left or self.parent.physics.touch.right then
					flip = true
				end
			elseif self.parent.direction == "left" then
				testX = self.parent.x - 4
				testY = self.parent.y + (self.parent.physics.offset.y) + (self.parent.physics.height / 2) + (self.parent.physics.velocity.y * (dt * speed))

				if self.parent.physics.touch.top or self.parent.physics.touch.bottom then
					flip = true
				end
			elseif self.parent.direction == "right" then
				testX = self.parent.x + self.parent.physics.width + self.parent.physics.offset.x + 4
				testY = self.parent.y + (self.parent.physics.offset.y) + (self.parent.physics.height / 2) + (self.parent.physics.velocity.y * (dt * speed))

				if self.parent.physics.touch.top or self.parent.physics.touch.bottom then
					flip = true
				end
			end


			local items, len = phys():queryPoint(testX, testY)

			if len == 0 then
				flip = true
			end

			if flip then
				self.dir = -self.dir

				if self.parent.direction == "up" or self.parent.direction == "down" then
					self.parent.sprite.flip.x = not self.parent.sprite.flip.x
					self.parent.physics.velocity.x = self.walkSpeed * self.dir
				else
					self.parent.sprite.flip.y = not self.parent.sprite.flip.y
					self.parent.physics.velocity.y = self.walkSpeed * self.dir
				end
			end
		end,
		takeHit = function(self)
			self.state:kill(self.parent)
		end,
		onBellRing = function(self)
			self.state:hurt(self.parent)
		end,
		on_hit = function(self, interactor)
			print "STUN"
		end
	}
end
