

return function()
	return {
		touched = false,
		awake = function(self)
			self.parent.y = self.parent.y - self.parent.sprite.height
			self.state = game:machine{
				initial = "idle",
				events = {
					{ name = "open", from = {"idle", "closed"}, to = "open" },
					{ name = "close", from = {"idle", "open"}, to = "closed"},
				},
				callbacks = {
					onclose = function(self, event, to, from, parent)
						local lc = game.context
						game.context = parent

						parent.physics.collides = true
						sfx "assets/sounds/door_close.wav"
						play_animation "closing"
						set_animation "closed"

						game.context = lc
					end,
					onopen = function(self, event, to, from, parent)
						game.save.flags.opened[f("%s#%s", game.room, parent.name)] = true

						local lc = game.context
						game.context = parent

						if parent.queue:isEmpty() then
							parent.physics.collides = false
							sfx "assets/sounds/door_open.wav"
							play_animation "opening"
							set_animation "open"
						end

						game.context = lc
					end
				}
			}

			if game.save.flags.opened[f("%s#%s", game.room, self.parent.name)] or self.parent.behind then
				self.state:open(self.parent)
			end
		end,
		draw = function(self)
			local lc = game:setLayer "ui_fg"

			if self.parent.combo and self.parent.combo ~= 0 then
				love.graphics.setFont(game.fonts.system)
				local t = f("%i / %i", state "combo", self.parent.combo)
				love.graphics.setColor(color "Black")
				love.graphics.rectangle("fill",
					self.parent.x - (game.fonts.system:getWidth(t) / 2) + game.camera.x + 4,
					self.parent.y + self.parent.animation.height + game.camera.y + 3,
					game.fonts.system:getWidth(t) + 4,
					game.fonts.system:getHeight() + 4
				)
				love.graphics.setColor(255, 255, 255, 255)

				local c = color "Red"

				if state "combo" < self.parent.combo / 3 then
					c = color "Red"
				elseif state "combo" < (self.parent.combo / 3) * 2 then
					c = color "Yellow"
				elseif state "combo" >= (self.parent.combo) then
					c = color "Green"
				end

				outline_print(t, self.parent.x - (game.fonts.system:getWidth(t) / 2) + game.camera.x + 6, self.parent.y + self.parent.animation.height + game.camera.y + 5, color "Black", c)
			end


			love.graphics.setCanvas(lc)
		end,
		collide = function(self, collision)
			if self.parent.behind then
				if collision.other.class == "player" then
					self.touched = true
					self.passedThrough = true
				end
			end
		end,
		update = function(self, dt, speed)
			if self.parent.behind then
				if self.state.current ~= "closed" then
					if self.passedThrough and not self.touched then
						self.state:close(self.parent)
					end
				end
			else
				if self.state:can "open" then
					local shouldOpen = {}

					if self.parent.combo and self.parent.combo ~= 0 then
						if self.parent.combo <= state "combo" then
							table.insert(shouldOpen, true)
							self.parent.combo = false
						else
							table.insert(shouldOpen, false)
							self.parent.animation:setAnimation "closed_combo"
						end
					end

					if self.parent.enemies then
						local enemiesLeft = false

						for index, entity in pairs(game:getEntities()) do
							if entity.behavior and entity.behavior:isEnemy() then
								enemiesLeft = true
							end
						end

						if enemiesLeft then
							self.parent.animation:setAnimation "closed_skull"
						else
							self.parent.enemies = false
						end
						table.insert(shouldOpen, not enemiesLeft)
					end

					if self.parent.targets and self.parent.targets ~= "" then
						local hitAll = true

						for index, target in pairs(self.parent.targets:separate(",")) do
							if not game:get(target).behavior:beenHit() then
								hitAll = false
							end
						end

						if not hitAll then
							self.parent.animation:setAnimation "closed_target"
						else
							self.parent.targets = false
						end

						table.insert(shouldOpen, hitAll)
					end

					if self.parent.targetClass and self.parent.targetClass ~= "" then
						table.insert(shouldOpen, (#getClass(self.parent.targetClass) == 0))
					end

					if self.parent.keys and self.parent.keys ~= "" then
						local hitAll = true

						for index, target in pairs(self.parent.keys:separate(",")) do
							if game:get(target) then
								hitAll = false
							end
						end

						if not hitAll then
							self.parent.animation:setAnimation "closed_key"
						else
							self.parent.keys = false
						end

						table.insert(shouldOpen, hitAll)
					end

					local so = true

					for index, key in pairs(shouldOpen) do
						if key == false then
							so = false
						end
					end

					if so and self.state:can("open") then
						self.state:open(self.parent)
					end
				end
			end

			self.touched = false
		end
	}
end
