--
--
--
--
--

return function()
	return {
		direction = "left",
		bind = true,

		directionOrder = {"left", "right"},
		directionIndex = 1,

		touchingPlayer = false,
		touchedPlayer = false,

		type = 1,

		speed = 150,
		awake = function(self)
			if not self.parent.type then
				self.parent.type = 1
			end

			self.parent.physics.width = self.parent.width
			self.parent.physics.height = self.parent.height

			self.direction = self.parent.direction

			if self.direction == "up" or self.direction == "down" then
				if self.direction == "up" then
					self.directionOrder = {"up", "down"}
				else
					self.directionOrder = {"down", "up"}
				end
			else
				if self.direction == "left" then
					self.directionOrder = {"left", "right"}
				else
					self.directionOrder = {"right", "left"}
				end
			end

			self.speed = self.parent.speed
			self.parent.sprite.slice = "left"

			self.parent.physics.collide = function(self, other)
				if other.type == "map" and self.type == 2 then
					return "cross"
				else
					return ((other.physics and other.physics.collides) or other.type == "map") and "slide" or "cross"
				end
			end
		end,
		update = function(self, dt, speed)

			local dir = game.definitions.directions[self.direction]
			self.parent.physics.velocity.x = dir.x * self.speed
			self.parent.physics.velocity.y = dir.y * self.speed

			if self.bind then
				if self.parent.x < -self.parent.animation.width
				or self.parent.x > game.roomWidth + self.parent.physics.width
				or self.parent.y > game.roomHeight + self.parent.physics.height
				or self.parent.y < -self.parent.physics.height then
					if self.parent.type == 1 then
						game:remove(self.parent)
					else
						if self.parent.dir == "left" then
							self.parent.x = 0
						elseif self.parent.dir == "right" then
							self.parent.x = game.roomWidth - 2
						elseif self.parent.dir == "up" then
							self.parent.y = game.roomHeight - 2
						elseif self.parent.dir == "down" then
							self.parent.y = 0
						end
					end
				end
			end

			if game.controls:pressed "a" then
				self.parent.physics.velocity.x = 0
				self.parent.physics.velocity.y = 0
			end

			if not self.touchingPlayer and self.touchedPlayer then
				if game:get "player" then
					game:get("player").behavior.onPlatform = false
					game:get("player").physics.baseVelocity = {x = 0, y = 0}
				end
			end

			self.touchedPlayer = self.touchingPlayer
			self.touchingPlayer = false
		end,
		draw = function(self)
			for x = 1, (math.floor(self.parent.physics.width / 8) - 2) do
				drawSlice(self.parent, "center", self.parent.x + (x * 8), self.parent.y)
			end
			drawSlice(self.parent, "right", self.parent.x + self.parent.physics.width - 8, self.parent.y)
		end,
		doesExplode = function(self)
			return false
		end,
		collide = function(self, collision, isOther)
			local other = isOther and collision.item or collision.other

			if other.type == "map" then
				if self.parent.type == 1 then
					self.directionIndex = self.directionIndex + 1
					if self.directionIndex > #self.directionOrder then
						self.directionIndex = 1
					end

					self.direction = self.directionOrder[self.directionIndex]
				end
			end

			if other.physics then
				if other.class == "player" then
					self.touchingPlayer = true
					other.behavior.onPlatform = true

					if (self.direction == "up" or self.direction == "down") then
						other.physics.velocity.y = self.direction == "down" and self.speed or -self.speed

						other.physics.baseVelocity.x = 0 --= self.direction == "down" and self.speed or -self.speed
					end

					if (self.direction == "left" or self.direction == "right") then
						other.physics.baseVelocity.x = self.direction == "right" and self.speed or -self.speed
						other.physics.baseVelocity.y = 0
					end
				end
			end
		end
	}
end
