return function ()
	return {
		state = "idle",
		moveSpeed = 120,
		airSpeed = 150,
		airAccel = 10,
		jumpSpeed = -300,
		rollSpeed = 300,
		rollTime = 0,
		minRollTime = 0.1,
		maxRollTime = 0.3,
		wallTime = 0,
		wallWindow = false,
		holdSide = false,
		doneHit = {},
		iframes = 0,
		bind = true,
		locked = false,
		invulnerable = false,
		onPlatform = false,

		jumping = false,
		swinging = false,

		awake = function(self, dt, speed)

			self.parent.physics._collide = self.parent.physics.collide
			self.parent.physics.collide = function(self, other)
				if other.class == "ball" then
					return "cross"
				elseif other.behavior and other.behavior:isEnemy() then
					if self.behavior.invulnerable then
						return "cross"
					else
						return "slide"
					end
				else
					return self.physics._collide(self, other)
				end
			end

			self.state = game:machine{
				parent = self.parent,
				events = {
					{name = 'hold_wall', from = {'jumping', 'falling'}, to = 'wall_hold'},
					{name = 'fall', from = {'wall_hold', 'idle', 'jumping'}, to = 'falling'},
					{name = 'land', from = {'jumping', 'wall_hold'}, to = 'idle'},
					{name = 'jump', from = {'idle', 'wall_hold'}, to = 'jumping'},
					{name = 'roll', from = {'idle'}, to = 'rolling'},
					{name = 'swing', from = {'idle', 'jumping', 'falling', 'rolling', 'wall_hold'}, to = 'swinging'},
					{name = 'idle', from = {'none', 'damaged', 'falling', 'swinging', 'rolling'}, to = 'idle'},
					{name = "damage", from = {'none', 'falling', 'swinging', 'wall_hold', 'idle', 'jumping'}, to = "damaged"},
					{name = "kill", from = {"damaged", "falling", "jumping", "wall_hold", "idle", "swinging"}, to = "killed"}
				},
				callbacks = {
					onkill = function(self, event, to, from, parent)
						game:remove(parent)
						game.targetContext = "game"
						sfx "assets/sounds/death.wav"
						fx("death", parent.center.x, parent.center.y, true)
						wait(0.5)
						code(function ()
							warp(game.lastUsedWarp)
						end)
						game.targetContext = "entity"
					end,
					onblink = function(self, event, to, from, parent)
						local balls = game:getEntities(function(entity) return entity.class == "ball" end)
						local ball = getNearest(balls, parent.x, parent.y)

						if ball then
							sfx "assets/sounds/blink.wav"
							fx("spawn", parent.x, parent.y)
							fx("spawn", ball.x, ball.y)

							parent.x = ball.x
							parent.y = ball.y
							parent.behavior.state:idle(parent)
						else
							sfx "assets/sounds/miss.wav"
						end
					end,
					ondamage = function(self, event, to, from, parent, dir)
						local lc = game.context
						game.context = parent
						if state "combo" == 0 then
							parent.behavior.state:kill(parent)
						else
							state("combo", 0)

							game.audio:play("assets/sounds/damage.wav", "sfx")
							parent.behavior.iframes = 1

							if dir == "left" then
								parent.physics.velocity.x = -400
							elseif dir == "right" then
								parent.physics.velocity.x = 400
							elseif dir == "up" then
								parent.physics.velocity.y = -400
							elseif dir == "down" then
								parent.physics.velocity.y = 400
							end

							parent.behavior.state:idle(parent)
						end
						game.context = lc
					end,
					onstatechange = function(self, event, to, from, parent)
						print(f("%s: %s -> %s", event, to, from))

						if event ~= "jump" and event ~= "swing" and event ~= "kill" then
							parent.queue:clear()
						end
					end,
					onleavejumping = function(self, event, to, from, parent)

					end,
					onfall = function(self, event, to, from, parent)
						if to == "jumping" then
							parent.queue:clear()
							parent.physics.gravity = {
								x = 0, y = 32
							}
						end
					end,
					onidle = function(self, event, to, from, parent)
						if to ~= "swinging" then
							parent.behavior.doneSwing = false
						end

						gravity {x = false, y = false}
					end,
					onjump = function(self, event, to, from, parent, wall)
						parent.physics.velocity.x = 0
						parent.physics.velocity.y = 0
						set_animation "jumping"
						sfx "assets/sounds/jump.wav"

						gravity{x = 0, y = 0}
						velocity{
							x = wall and (parent.sprite.flip.x and parent.behavior.airSpeed * -1 or parent.behavior.airSpeed * 1) or nil,
							y = parent.behavior.jumpSpeed
						}
						if wall then
							lock()
							wait(0.1)
							gravity{x = 0, y = 32}
							unlock()
						else
							wait(0.25)
							gravity{x = 0, y = 32}
						end

					end,
					onenterwall_hold = function(self, event, to, from, parent)
						parent.behavior.doneSwing = false
						parent.physics.gravity.y = 4
						--parent.physics.velocity.y = 0

						if parent.physics.touch.left then
							parent.behavior.holdSide = "left"
							parent.sprite.flip.x = true
						elseif parent.physics.touch.right then
							parent.behavior.holdSide = "right"
							parent.sprite.flip.x = false
						end

						printf("holding wall side: %s", parent.behavior.holdSide)
					end,
					onleavewall_hold = function(self, event, to, from, parent)
						parent.behavior.wallTime = 0
						parent.physics.gravity.y = false
						parent.behavior.holdSide = false
					end,
					onland = function(self, event, to, from, parent)
						parent.behavior.doneSwing = false
						game.audio:play("assets/sounds/land.wav", "sfx")
					end,
					onswing = function(self, event, to, from, parent)
						local balls = getClass("ball")
						if #balls == 0 and not parent.behavior.doneSwing then
							local ball = game:add("ball")
							local spawnX = parent.x + parent.physics.offset.x + (parent.physics.width / 2) - (ball.physics.offset.x + ball.physics.width / 2) + 4
							local spawnY = parent.y + parent.physics.offset.y + (parent.physics.height / 2) - (ball.physics.offset.y + ball.physics.height / 2)
							ball.x = spawnX
							ball.y = spawnY

							local h = parent.sprite.flip.x and -1 or 1
							local v = -0.25

							if game.controls:down "up" then
								v = -1
							elseif game.controls:down "down" then
								v = 1
							else
								v = 0
							end

							if game.controls:down "left" then
								h = -1
							elseif game.controls:down "right" then
								h = 1
							else
								if v ~= 0 then
									h = 0
								else
									h = parent.sprite.flip.x and 1 or - 1
								end
							end

							local baseVel = track(game.definitions.comboSpeed, state "combo")
							local vel = math.rotate(baseVel, baseVel, math.angle(0, 0, h, v) - math.rad(45))

							ball.physics.velocity.x = vel.x
							ball.physics.velocity.y = vel.y

							if to == "rolling" or to == "wall_hold" then
								ball.charged = true
							end

							sfx "assets/sounds/swing.wav"
							local h,v = game.controls:get "directions"
							local dir = "side"
							if math.abs(v) > math.abs(h) then
								if v < 0 then
									dir = "up"
								else
									dir = "down"
								end
							end
							if not parent.behavior.onPlatform then
								gravity {x = 0, y = 0}
								velocity {x = 0, y = 0}
							end
							play_animation ("swing_" .. dir)
							gravity {x = false, y = false}
							parent.behavior.doneSwing = true
						end
						code(function(self)
							self.parent.behavior.doneHit = {}
							self.parent.behavior.state:idle(self.parent)
						end)
					end,
					onroll = function(self, event, from, to, parent)
						print "rolling"
						local h,v = game.controls:get "directions"

						game.audio:play("assets/sounds/roll.wav", "sfx")
						if h == 0 then
							--h = parent.sprite.flip.x and 1 or -1
						end
						parent.animation:setAnimation "jumping"
						parent.physics.velocity.x = math.sign(h) * parent.behavior.rollSpeed + parent.physics.baseVelocity.x
						parent.physics.velocity.y = 0 + parent.physics.baseVelocity.y
					end,
					onleaverolling = function(self, event, to, from, parent)
						parent.behavior.rollTime = 0
						parent.behavior.invulnerable = false
					end,
					onleaveswinging = function(self, event, to, from, parent)
						if parent.behavior.hitBall then
							state("combo", state "combo" + 1)
						else
							if state "combo" > 3 then
								game.audio:play("assets/sounds/miss.wav", "sfx")
							end

							state("combo", 0)
						end

						parent.behavior.hitBall = false
					end
				}
			}
		end,

		update = function(self, dt, speed)

			if self.bind then
				if self.parent.x < -self.parent.animation.width
				or self.parent.x > game.roomWidth + self.parent.physics.width
				or self.parent.y > game.roomHeight + self.parent.physics.height then
					print "Killed for out of bounds."
					self.state:kill(self.parent)
				end
			end

			local hitbox = self.parent.sprite.slices.hitbox.frames[self.parent.animation.index]
			local flip = self.parent.sprite.flip.x
			local lastphys = {
				x = self.parent.physics.offset.x,
				y = self.parent.physics.offset.y,
				w = self.parent.physics.width,
				h = self.parent.physics.height
			}

			self.parent.physics.width = hitbox.bounds.w
			self.parent.physics.offset.x = hitbox.bounds.x
			self.parent.physics.offset.y = hitbox.bounds.y
			self.parent.physics.height = hitbox.bounds.h

			if not game.locked and not self.locked then
				if game.controls:pressed "select" then
					chooseWarp()
				end

				if self.state.current == "none" then
					self.state:idle(self.parent)
				else
					if self[self.state.current] then
						self[self.state.current](self, dt, speed)
					end
				end
			end
		end,
		rolling = function(self, dt, speed)
			self.rollTime = self.rollTime + (dt * speed)
			self.invulnerable = true

			if self.rollTime > self.minRollTime then
				if not game.controls:down "b" or self.rollTime > self.maxRollTime then
					local items, len = phys():queryRect(self.parent.x + self.parent.physics.offset.x, self.parent.y + self.parent.physics.offset.y, self.parent.physics.width, self.parent.physics.height)

					local canUnroll = true

					for index, entity in pairs(items) do
						if entity.type ~= "map" and entity.type ~= "shield" then
							if entity.class ~= "player" and entity.physics.collides then
								canUnroll = false
								printf("Overlap: %s", entity.name)
							end
						end
					end

					if canUnroll ~= false then
						self.state:idle(self.parent)
					else
						print "Can't unroll."
					end
				else
					if game.controls:down "x" then
						self.state:swing(self.parent)
					end
				end
			end
		end,
		idle = function(self, dt, speed)
			local h, v = game.controls:get "directions"

			if not self.parent.physics.touch.bottom and not self.onPlatform then
				self.state:fall(self.parent)
			else
				if self.parent.physics.touch.bottom then
					self.doneSwing = false
				end
			end

			if game.controls:pressed "a" then
				self.state:jump(self.parent)
			elseif game.controls:pressed "b" then
				self.state:roll(self.parent)
			elseif game.controls:pressed "x" then
				self.state:swing(self.parent)
			else
				if math.abs(h) > 0 then
					self.parent.physics.velocity.x = self.moveSpeed * math.sign(h) +  self.parent.physics.baseVelocity.x
					self.parent.animation:setAnimation "walk"

					if h > 0 then
						self.parent.sprite.flip.x = true
					else
						self.parent.sprite.flip.x = false
					end
				else
					self.parent.animation:setAnimation "idle"
					self.parent.physics.velocity.x = self.parent.physics.baseVelocity.x
				end
			end
		end,
		falling = function(self, dt, speed)
			local h, v = game.controls:get "directions"
			if math.abs(h) > 0 then
				self.parent.physics.velocity.x = self.moveSpeed * math.sign(h)

				if h > 0 then
					self.parent.sprite.flip.x = true
				else
					self.parent.sprite.flip.x = false
				end
			end

			if game.controls:pressed "x" then
				self.state:swing(self.parent)
			end

			self.parent.animation:setAnimation "jumping"

			if self.parent.physics.touch.bottom or self.onPlatform then
				self.state:idle(self.parent)
			elseif (self.parent.physics.touch.left or self.parent.physics.touch.right) then
				self.state:hold_wall(self.parent)
			end

		end,
		wall_hold = function(self, dt, speed)
			self.wallTime = self.wallTime + (dt * speed)
				local h, v = game.controls:get "directions"
			self.parent.animation:setAnimation "wall_hold"

			if self.wallTime > 0.1 or self.parent.physics.velocity.y > 0 then
				self.parent.physics.velocity.y = 0
			end

			self.parent.physics.gravity.y = self.parent.physics.gravity.y + ((dt * speed) * 10)

			if not self.parent.physics.touch[self.holdSide] then
				self.state:fall(self.parent)
			else
				self.parent.physics.velocity.x = self.airSpeed * h

				if game.controls:pressed "a" then
					self.state:jump(self.parent, "wall")
				end

				if game.controls:pressed "x" then
					self.state:swing(self.parent)
				end
			end

			if self.parent.physics.touch.bottom then
				self.state:land(self.parent)
			end
		end,
		collide = function(self, collision)
			if self.iframes <= 0 then
				if collision.other.type == "map" and collision.other.class == "spike" then
					self.state:kill(self.parent)
				end
			end
		end,
		swinging = function(self, dt, speed)
			local swingBox = self.parent.sprite.slices.racket.frames[self.parent.animation.index].bounds
			local startX = self.parent.x + swingBox.x

			if self.parent.sprite.flip.x then
				startX = self.parent.x + self.parent.animation.width - swingBox.x - swingBox.w
			end

			local items, len = game:getSystem("physics").world:queryRect(startX, self.parent.y + swingBox.y, swingBox.w, swingBox.h)

			if len > 0 then
				for index, entity in pairs(items) do
					if entity ~= self.parent then
						local dh = false
						for index, eb in pairs(self.doneHit) do
							if eb == entity then
								dh = true
							end
						end

						if not dh then
							if entity.behavior then
								entity.behavior:on_hit(self.parent)
								table.insert(self.doneHit, entity)

								if entity.class == "ball" then
									self.hitBall = true
								end
							end
						end
					end
				end
			end
		end,
		draw = function(self)
			if game.debugMode then
				local lc = game:setLayer "debug"

				if self.state.current == "swinging" then
					local swingBox = self.parent.sprite.slices.racket.frames[self.parent.animation.index].bounds
					local startX = self.parent.x + swingBox.x

					if self.parent.sprite.flip.x then
						startX = self.parent.x + self.parent.animation.width - swingBox.x - swingBox.w
					end

					love.graphics.rectangle("fill", startX, self.parent.y + swingBox.y, swingBox.w, swingBox.h)
				end

				love.graphics.setCanvas(lc)
			end
		end,
		on_hit = function(self, hitter)
			self.state:fall(self.parent)
		end,
		jumping = function(self, dt, speed)

			local h, v = game.controls:get "directions"
			if math.abs(h) > 0 then
				self.parent.physics.velocity.x = (self.moveSpeed * math.sign(h))

				if h > 0 then
					self.parent.sprite.flip.x = true
				else
					self.parent.sprite.flip.x = false
				end
			end

			if (self.parent.physics.touch.left or self.parent.physics.touch.right) then
				self.state:hold_wall(self.parent)
			end

			if game.controls:released "a" or self.parent.physics.touch.top then
				self.state:fall(self.parent)
			end

			if game.controls:pressed "x" then
				self.state:swing(self.parent)
			end

			if self.parent.queue:isEmpty() then
				if self.parent.physics.touch.bottom then
					self.state:land(self.parent)
				end
			end
		end
	}
end
