

return function()
	return {
		dir = -1,
		vision = 500,
		awake = function(self)
			if self.parent.dir == "right" then
				self.parent.sprite.flip.x = true
				self.dir = 1
			end

			local hitbox = self.parent.sprite.slices.hitbox.frames[self.parent.animation.index]
			local flip = self.parent.sprite.flip.x

			self.parent.physics.width = hitbox.bounds.w
			self.parent.physics.offset.x = flip and self.parent.animation.width - hitbox.bounds.w - hitbox.bounds.x or hitbox.bounds.x
			self.parent.physics.offset.y = hitbox.bounds.y
			self.parent.physics.height = hitbox.bounds.h

			self.parent.y = self.parent.y - 8
			self.state = game:machine{
				initial = "idle",
				events = {
					{name = "idle", from = "firing", to = "idle"},
					{name = "fire", from = "idle", to = "firing"},
					{name = "kill", from = {"idle", "firing"}, to = "dead"}
				},
				callbacks = {
					onfire = function(self, event, from, to, parent)
						local lc = game.context
						game.context = parent
						for i = 1, parent.amount do
							play_animation "shoot"
							code(function(self, options)
								game:add("enemies/bone", {
									x = self.parent.x + self.parent.physics.offset.x + 4,
									y = self.parent.y + self.parent.physics.offset.y,
									dir = self.parent.behavior.dir,
									physics = {
										velocity = {
											y = 0,
											x = self.parent.speed * self.parent.behavior.dir
										},
										gravity = {
											x = 0,
											y = 0
										}
									}
								})
							end)
							play_animation "shoot_back"
							set_animation "idle"
							wait(parent.buffer)
						end
						set_animation "idle"
						wait(parent.wait)
						code(function(self)
							self.parent.behavior.state:idle(self.parent)
						end)

						game.context = lc
					end,

					onkill = function(self, event, to, from, parent)
						local lc = game.context
						game.context = parent
						parent.queue:clear()
						velocity {x = 0, y = 0}
						fx("explode", parent.x + 12, parent.y + 24, true)
						removeSelf()
						game.context = lc
					end
				}
			}
			self.parent.physics._collide = self.parent.physics.collide
			self.parent.physics.collide = function(self, other)
				if other.class == "ball" then
					return "cross"
				elseif other.class == "player" then
					if other.behavior.invulnerable then
						return "cross"
					else
						return "slide"
					end
				else
					return self.physics._collide(self, other)
				end
			end

			local lc = game.context
			game.context = self.parent
			wait(self.parent.init_delay)

			game.context = lc
		end,
		update = function(self, dt, speed)
			if not game.locked then
				local hitbox = self.parent.sprite.slices.hitbox.frames[self.parent.animation.index]
				local flip = self.parent.sprite.flip.x

				self.parent.physics.width = hitbox.bounds.w
				self.parent.physics.offset.x = flip and self.parent.animation.width - hitbox.bounds.w - hitbox.bounds.x or hitbox.bounds.x
				self.parent.physics.offset.y = hitbox.bounds.y
				self.parent.physics.height = hitbox.bounds.h

				if self[self.state.current] then
					self[self.state.current](self, dt, speed)
				end
			end
		end,
		isEnemy = function(self)
			return true
		end,
		collide = function(self, collision)
			if collision.other.class == "ball" then
				local hitEnemies = collision.other.behavior.hitEnemies
				local doHit = true
				for index, enemy in pairs(hitEnemies) do
					if enemy == self.parent then
						doHit = false
					end
				end

				if doHit then
					collision.other.behavior.shouldExplode = true
					self:takeHit()
					table.insert(collision.other.behavior.hitEnemies, self.parent)
					print "HURT"
				else

				end
			end
		end,
		idle = function(self, dt, speed)
			self.state:fire(self.parent)
		end,
		takeHit = function(self)
			self.state:kill(self.parent)
		end,
		onBellRing = function(self)

		end,
		on_hit = function(self, interactor)
			print "STUN"
		end
	}
end
