

return function()
	return {
		dir = -1,
		vision = 500,
		pattern = 2,
		walkSpeed = 64,
		shield = {},
		awake = function(self)
			self.parent.y = self.parent.y - 24
			local shieldBox = self.parent.sprite.slices.shield.frames[self.parent.animation.index].bounds
			self.shield = createShield(self.parent, shieldBox)

			self.state = game:machine{
				initial = "idle",
				events = {
					{name = "idle", from = {"hurt", "walk", "stun", "attacking"}, to = "idle"},
					{name = "walk", from = {"idle", "stun", "attacking"}, to = "walk"},
					{name = "stun", from = {"idle", "attacking"}, to = "stun"},
					{name = "hurt", from = {"idle", "walk", "attacking", "stun"}, to = "hurt"},
					{name = "attack", from = {"idle", "walk"}, to = "attacking"},
					{name = "kill", from = {"idle", "walk", "attacking", "hurt", "stun"}, to = "dead"}
				},
				callbacks = {
					onstatechange = function(self, event, to, from, parent)

					end,
					onattack = function(self, event, to, from, parent)
						play_animation "windup"
						play_animation "strike"
						play_animation "drawback"
						set_animation "idle"
						code(function (self)
							self.parent.behavior.state:idle(self.parent)
						end)
						game.context = lc
					end,
					onhurt = function(self, event, to, from, parent)
						local lc = game.context
						game.context = parent

						play_animation "hurt"
						set_animation "idle"
						code(function(self)
							self.parent.behavior.state:idle(self.parent)
						end)

						game.context = lc
					end,
					onkill = function(self, event, to, from, parent)
						local lc = game.context
						game.context = parent
						parent.queue:clear()
						velocity {x = 0, y = 0}
						play_animation "hurt"
						fx("explode",
							parent.x + parent.physics.offset.x + (parent.physics.width / 2),
							parent.y + parent.physics.offset.y + (parent.physics.height / 2), true)
						removeSelf()
						game.context = lc
					end
				}
			}

			self.parent.physics._collide = self.parent.physics.collide
			self.parent.physics.collide = function(self, other)
				if other.class == "ball" then
					return "slide"
				elseif other.class == "player" then
					if other.behavior.invulnerable then
						return "cross"
					else
						return "slide"
					end
				else
					return self.physics._collide(self, other)
				end
			end
		end,
		update = function(self, dt, speed)

			phys():update(self.shield,
				self.parent.x + self.parent.sprite.slices.shield.frames[self.parent.animation.index].bounds.x,
				self.parent.y + self.parent.sprite.slices.shield.frames[self.parent.animation.index].bounds.y)

			local hitbox = self.parent.sprite.slices.hitbox.frames[self.parent.animation.index]
			local flip = self.parent.sprite.flip.x

			self.parent.physics.width = hitbox.bounds.w
			self.parent.physics.offset.x = flip and self.parent.animation.width - hitbox.bounds.w - hitbox.bounds.x or hitbox.bounds.x
			self.parent.physics.offset.y = hitbox.bounds.y
			self.parent.physics.height = hitbox.bounds.h

			if self[self.state.current] then
				self[self.state.current](self, dt, speed)
			end
		end,
		isEnemy = function(self)
			return true
		end,
		collide = function(self, collision, isOther )
			local other = isOther and collision.item or collision.other

			if other.class == "ball" then
				local hitEnemies = other.behavior.hitEnemies
				local doHit = true
				for index, enemy in pairs(hitEnemies) do
					if enemy == self.parent then
						doHit = false
					end
				end

				if doHit then
					other.behavior.shouldExplode = true
					self:takeHit()
					table.insert(other.behavior.hitEnemies, self.parent)
					print "HURT"
				else

				end
			end
		end,
		idle = function(self, dt, speed)
			self.state:walk(self.parent)
		end,
		walk = function(self, dt, speed)
			self.shield.active = true
			local vision = self.parent.sprite.slices.vision.frames[self.parent.animation.index].bounds
			local items, len = phys():queryRect(
				self.parent.x + vision.x,
				self.parent.y + vision.y,
				vision.w, vision.h
			)

			local attack = false

			if len > 0 then
				for index, item in pairs(items) do
					if item.class == "player" then
						attack = true
					end
				end
			end

			if attack then
				self.state:attack(self.parent)
			end
		end,
		attacking = function(self, dt, speed)
			self.shield.active = false
			local spear = self.parent.sprite.slices.spear.frames[self.parent.animation.index].bounds

			if self.parent.animation.current == "strike" then
				local items, len = phys():queryRect(self.parent.x + spear.x, self.parent.y + spear.y, spear.w, spear.h)

				for index, i in pairs(items) do
					if i.class == "player" and not i.behavior.invulnerable then
						print "Killed by spear."
						i.behavior.state:kill(i)
					end
				end
			end
		end,
		takeHit = function(self)
			self.state:kill(self.parent)
		end,
		onBellRing = function(self)
			self.state:hurt(self.parent)
		end,
		on_hit = function(self, interactor)
			print "STUN"
		end,
		doesExplode = function(self)
			return false
		end,
		remove = function(self)
			if phys():hasItem(self.shield) then
				phys():remove(self.shield)
			end
		end
	}
end
