--
--
-- Debug render script.
--
--

love.graphics.setFont(game.fonts[game.defaultFont])

local debugStrings = {}

table.insert(debugStrings, f("time: %i", game.runtime))

table.insert(debugStrings, f("scene: %s", game.activeScene))
table.insert(debugStrings, f("locked: %s", game.locked))
table.insert(debugStrings, f("cutscene: %s", game.cutscene.active))
table.insert(debugStrings, f("room: %s", game.room))
table.insert(debugStrings, f("tab: %s", game.tab))
table.insert(debugStrings, f("camera pos: x = %i, y = %i", game.camera.x, game.camera.y))

local h,v = game.controls:get "directions"
table.insert(debugStrings, f("controls: h: %f, v: %f", h, v))

table.insert(debugStrings, "")
table.insert(debugStrings, "tokens")

if game.save.tokens[game.room] then
	for name, token in pairs(game.save.tokens[game.room]) do
		table.insert(debugStrings, f("%s: %s", name, token))
	end
else
	table.insert(debugStrings, "(no tokens)")
end

table.insert(debugStrings, "")
table.insert(debugStrings, "temporary flags")

if #game.tempFlags > 0 then
	for name, token in pairs(game.tempFlags) do
		table.insert(debugStrings, f("%s: %s", name, token))
	end
else
	table.insert(debugStrings, "(no temp flags)")
end

table.insert(debugStrings, "")
table.insert(debugStrings, "queues")
table.insert(debugStrings, f("target context: %s", game.targetContext))
table.insert(debugStrings, f("game[%s]: %i", game.queue:getActiveID(), #game.queue.queue))
table.insert(debugStrings, f("scene[%s]: %i", game:scene().queue:getActiveID(), #game:scene().queue.queue))


for index, entityName in pairs(game:scene().entityOrder) do
	if game:get(entityName).queue and #game:get(entityName).queue.queue > 0 then
		table.insert(debugStrings, f("%s[%s]: %i", entityName, game:get(entityName).queue:getActiveID(), #game:get(entityName).queue.queue))
	end
end
table.insert(debugStrings, "")
table.insert(debugStrings, "behaviors")
for index, entityName in pairs(game:scene().entityOrder) do
	if game:get(entityName).behavior then
		local entity = game:get(entityName)
		table.insert(debugStrings, f("%s: %s", entityName, type(entity.behavior.state) == "table" and entity.behavior.state.current or entity.behavior.state))
	end
end
table.insert(debugStrings, "")
table.insert(debugStrings, "physics")
for index, entityName in pairs(game:scene().entityOrder) do
	if game:get(entityName).physics then
		local entity = game:get(entityName)
		if entity.physics.velocity.x ~= 0 or entity.physics.velocity.y ~= 0 then
			table.insert(debugStrings, f("%s: x%i, y%i", entityName, entity.physics.velocity.x, entity.physics.velocity.y))
		end
	end
end

table.insert(debugStrings, "")
table.insert(debugStrings, "state")
for key, value in pairs(game.state) do
	table.insert(debugStrings, f("%s: %s", key, value))
end


if game.debugMode == 2 then
local r,g,b,a = love.graphics.getColor()

local yIndex = 0
local yInc = 12
love.graphics.setFont(game.fonts.system)
for index, str in pairs(debugStrings) do
	love.graphics.setColor(0, 0, 0)
	local w = game.fonts.system:getWidth(str)
	love.graphics.rectangle("fill", 0, 2 + yIndex * yInc, w + 4, yInc + 2)
	outline_print(str, 2, 6 + yIndex * yInc)
	yIndex = yIndex + 1
end

love.graphics.setColor(r,g,b,a)
end
