--
--
--
--
--
--

--
-- Sets an entity to be active. Any active-entity commands will be evaluated with this entity as the context.
-- ``active "player"``
--
function particleSource(particle)
	local particles = love.graphics.newParticleSystem(love.graphics.newImage(particle), 5000)
	particles:setParticleLifetime(0.1, 0.5)
	particles:setLinearAcceleration(-150, -150, 150, 150)
	particles:setEmissionRate(0)
	particles:start()

	return particles
end

function state(key, value)
	if value then
		game.state[key] = value
	else
		return game.state[key]
	end
end

function onscreen(x, y)
	local x, y = game.camera:getScreenPosition(x, y)

	return (x >= 0 and x <= game.width) and (y >= 0 and y <= game.height)
end

function tween_curve(curve, t)
	local points = {}
	local p = {}

	for i = 1, #curve do
		table.insert(p, curve[i])
		if #p == 2 then
			table.insert(points, p)
			p = {}
		end
	end

	for index, point in pairs(points) do
		tween_to({x = point[1], y = point[2], t = t})
	end
end

function hold()
	game:findQueue({
		id = "hold",
		context = "any",
		type = "async",
		start = function(self)

			if self.parent then
				printf("Entity %s is holding", self.parent.name)
			else
				print("Holding")
			end
		end,
		update = function(self, dt, speed)
			print "Still holding"
		end,
		finish = function(self, dt, speed)

		end
	})
end

function shake(length, intensity, hold)
	if hold then
		game:findQueue({
			id = "shake",
			context = "any",
			type = "async",
			length = length,
			intensity = intensity,
			start = function(self)
				game.camera:shake(self.length, self.intensity)
			end,
			update = function(self, dt, speed)
				self.finished = not game.camera.shaking
			end
		})
	else
		game:findQueue({
			id = "shake",
			context = "any",
			type = "instant",
			length = length,
			intensity = intensity,
			run = function(self)
				game.camera:shake(self.length, self.intensity)
			end
		})
	end
end
function flag(key, value)
	if value then
		game.save.flags[key] = value
	else
		return game.save.flags[key]
	end
end

function temp(key, value)
	if value then
		game.tempFlags[key] = value
	else
		return game.tempFlags[key]
	end
end

function token(key, value)
	if value then
		printf("Setting token %s:%s to %s", game.room, key, value)
	end

	if not game.save.tokens[game.room] then
		game.save.tokens[game.room] = {}
	end

	if value then
		game.save.tokens[game.room][key] = value
	else
		return game.save.tokens[game.room][key]
	end
end

function waitForEmptyQueue(targets)
	game:findQueue{
		id = "waitForQueue",
		context = "entity",
		type = "async",
		targets = targets or {},
		start = function(self)
		end,
		update = function(self, dt, speed)
			self.finished = areQueuesEmpty(self.targets)
		end
	}
end

function camera(x, y, async)
	game:findQueue({
		id = "camera_pos",
		context = "any",
		type = async and "async" or "instant",
		x = x, y = y,
		run = function(self)
			game.camera:setPosition(self.x, self.y)
		end,
		start = function(self)
			game.camera:setPosition(self.x, self.y)
		end,
		update = function(self, dt, speed)
			if game.camera.x == self.x and game.camera.y == self.y then
				self.finished = true
			end
		end
	})
end

function flip(x, y)
	game:findQueue({
		id = "flip",
		context = "entity",
		type = "instant",
		x = x, y = y,
		run = function(self)
			if not self.parent.sprite then
				error "flip() called on entity with no sprite."
			end

			if self.x == "toggle" then
				self.parent.sprite.flip.x = not self.parent.sprite.flip.x
			else
				self.parent.sprite.flip.x = self.x
			end
			if self.y == "toggle" then
				self.parent.sprite.flip.y = not self.parent.sprite.flip.y
			else
				self.parent.sprite.flip.y = self.y
			end
		end
	})
end

function import(validate, process)
	game:findQueue({
		id = "import",
		context = "game",
		type = "async",
		validate = validate or function(file) return true end,
		process = process or save_file,
		start = function(self)
			self.notice = game:add("ui/window", {text = "Waiting on import...\n(Drag and drop onto the game.)"})
		end,
		update = function(self, dt, speed)
			if not game:get(self.notice.name) then
				notice "Import cancelled."
				self.finished = true
			end

			if game.dropped then
				if self.validate(game.dropped) then
					printf("Importing %s...", game.dropped:getFilename())
					local result = self.process(game.dropped)

					if result then
						notice("File imported!")
					else
						notice(string.format("Import failed."))
					end

					game.dropped = false
					self.finished = true
				else
					notice "Invalid file presented."
					game.dropped = false
					self.finished = true
				end
			end
		end
	})
end

function tab(t)
	game:findQueue({
		id = "tab",
		context = "scene",
		type = "instant",
		tab = t,
		run = function(self)
			game.queuedTab = self.tab
		end,
	})
end

function wipeToRoom(r)
	game.targetContext = "scene"
	wipe(0)
	brownout()
	room(r)
	brownin()
	wipe(1)
end

function brownout(speed)
	game:findQueue({
		speed = speed or 1,
		id = "brownout",
		context = "any",
		type = "async",
		start = function(self)
			game.brownout = 0
		end,
		update = function(self, dt, speed)
			game.brownout = game.brownout + ((dt * speed) * self.speed)

			if game.brownout > 1 then
				game.brownout = 1
				self.finished = true
			end
		end
	})
end

function brownin(speed)
	game:findQueue({
		id = "brownin",
		speed = speed or 1,
		context = "any",
		type = "async",
		start = function(self)
			game.brownout = 1
		end,
		update = function(self, dt, speed)
			game.brownout = game.brownout - ((dt * speed) * self.speed)

			if game.brownout < 0 then
				game.brownout = 0
				self.finished = true
			end
		end
	})
end

function wipe(state, onFinish, doOnFinish)
	game:findQueue({
		id = "wipe",
		context = "scene",
		type = "async",
		state = state,
		onFinish = onFinish ~= nil and onFinish or true,
		start = function(self)
			self.cover = game:add("cover", {state = self.state, removeOnFinish = self.onFinish})
		end,
		update = function(self, dt, speed)
			if self.cover.removeOnFinish then
				self.finished = not game:get(self.cover.name)
			end
		end,
		finish = doOnFinish
	})
end

function blackout()
	game:findQueue({
		id = "blackout",
		context = "scene",
		type = "instant",
		run = function(self)
			game.blackout = true
		end
	})
end

function blackin()
	game:findQueue({
		id = "blackout",
		context = "scene",
		type = "instant",
		run = function(self)
			game.blackout = false
		end
	})
end

function conversation(id, r)
	if r == nil then
		r = false
	end

	if love.filesystem.exists(string.format("assets/conversations/%s.json", id)) then
		local conversation = game.json.decode(love.filesystem.read(string.format("assets/conversations/%s.json", id)))

		local nodes = {}

		for index, node in pairs(conversation) do
			local text = nil
			local script = {}
			local buttons = {}
			local tags = {}
			local portrait = false

			for index, tag in pairs(node.tags:separate(',')) do
				local bites = tag:separate ':'
				local tag = bites[1]
				local value = bites[2]

				tags[tag] = value
			end

			for index, line in pairs(node.body:separate("\n")) do
				if line:sub(1, 2) == "[[" and line:sub(#line - 1, #line) == "]]" then
					local bites = line:sub(3, #line)
					bites = bites:sub(1, #bites - 2)
					bites = bites:separate("|")
					table.insert(buttons, {
						label = bites[1],
						next = bites[2],
						onclick = function(self)
							local nextNode = self.nodes[self.choices[self.selectedChoice].next]

							if not nextNode then
								return false
							end

							game.queue.queue.insert = 1

							for index, script in pairs(nextNode.script) do
								loadstring(script)()
							end

							game.queue.queue.insert = false

							if nextNode.queuedText ~= nil or nextNode.choices ~= nil then
								window(nextNode)
							end
						end
					})
				else
					if line ~= "" then
						text = (text or "") .. line .. "\n"
					end
				end
			end

			if node.title ~= "Template" then
				nodes[node.title] = tags
				nodes[node.title].nodes = nodes
				nodes[node.title].queuedText = text and queueText(text) or nil
				nodes[node.title].script = script
				nodes[node.title].choices = #buttons > 0 and buttons or nil
			else
				nodes[node.title] = tags
			end
		end

		if nodes.Template then
			for name, node in pairs(nodes) do
				if name ~= "Template" then
					nodes[name] = table.merge(nodes.Template, node)
				end
			end
		end

		if not r then
			if nodes.Start == nil then
				error(string.format("Conversation %s has no starting node", id))
			else
				game.targetContext = "scene"
				window(nodes.Start)
			end
		else
			return nodes
		end
	else
		error(f("Attempted to start conversation that doesn't exist: %s", id))
	end
end

function fx(name, x, y, instant)
	game:findQueue({
		id = "fx:" .. name,
		context = "any",
		type = instant and "instant" or "async",
		name = name,
		x = x,
		y = y,
		run = function(self)
			game:add("sfx", {
				x = self.x,
				y = self.y,
				sprite = {
					sprite = string.format("assets/sprites/sfx/%s.png", self.name)
				},
				animation = {
					schema = string.format("assets/sprites/sfx/%s.json", self.name)
				},
			})
		end,
		start = function(self)
			self.entity = game:add("sfx", {
				x = self.x,
				y = self.y,
				sprite = {
					sprite = string.format("assets/sprites/sfx/%s.png", self.name)
				},
				animation = {
					schema = string.format("assets/sprites/sfx/%s.json", self.name)
				},
			})
			self.entity.animation.loop = false
		end,
		update = function(self, dt, speed)
			self.finished = self.entity.animation.finished
		end,
		finish = function(self)
			game:remove(self.entity)
		end
	})
end

function notice(text)
	if type(text) == "string" then
		text = {text = formatText(text)}
	end

	game:findQueue({
		id = "notice",
		context = "scene",
		type = "async",
		text = text,
		start = function(self)
			self.box = game:add("ui/window", self.text)
			game.locked = true
		end,
		update = function(self, dt, speed)
			self.finished = not game:get(self.box.name)
		end,
		finish = function(self)
			game.locked = false
		end
	})
end

function window(options)
	if not options.text and not options.queuedText and not options.portrait and not options.choices then
		error "Window called with no children."
	end

	game:findQueue({
		id = "window",
		context = "any",
		type = "async",
		options = options,
		start = function(self)
			self.window = game:add("ui/window", self.options)
		end,
		update = function(self, dt, speed)
			self.finished = not game:get(self.window.name)
		end
	})
end

function choose(options)
	game:findQueue({
		id = "choose",
		context = "scene",
		type = "async",
		options = options,
		start = function(self)
			if self.options.buttons then
				self.options.choices = self.options.buttons
			end

			self.box = game:add("ui/window", self.options)
			game.locked = true
		end,
		update = function(self, dt, speed)
			self.finished = not game:get(self.box.name)
		end,
		finish = function(self)
			game.locked = false
		end
	})
end

function speak(text)
	game:findQueue({
		id = "speak",
		context = "entity",
		type = "async",
		text = text,
		start = function(self)
			local parent = self.parent

			x = parent.x
			y = parent.y

			self.speech = game:add("ui/speech", {x = x, y = y, text = self.text, parent = parent})
		end,
		update = function(self, dt, speed)
			self.finished = not game:get(self.speech.name)
		end
	})
end

function dialog(name, text)
	if text == nil and game.speaker then
		text = name
		name = game.speaker
	end

	game:findQueue({
		id = "dialogue",
		context = "scene",
		type = "async",
		text = text,
		npc = name,
		start = function(self)
			self.box = game:add("ui/window", {npc = self.npc, text = self.text})
		end,
		update = function(self, dt, speed)
			self.finished = not game:get(self.box.name)
		end
	})
end

function active(entity)
	print "Warning: Calling deprecated event 'active', no effect will be had."
	return false
	--[[
	game:findQueue({
		e = entity,
		context = "scene",
		type = "async",
		id = "active",
		run = function(self)
			game.activeEntity = self.e
		end
	})]]
end

function room(r)
	game:findQueue({
		id = "room",
		context = "scene",
		type = "instant",
		room = r,
		run = function(self)
			game.mapMode = "room"
 			load_room(self.room)
		end,
	})
end

function warp(target)
	local ls = game.targetContext
	game.targetContext = "scene"
	lock()
	brownout(5)
	game:findQueue({
		id = "warp",
		context = "any",
		type = "instant",
		target = target,
		run = function(self)
			local parts = target:separate('#')
			if parts[2] then
				printf("Warping to room %s, target %s", parts[1], parts[2])

				warp_to(parts[1], parts[2])
			else
				error "No target entity defined."
			end
		end
	})
	brownin(5)
	unlock()
	game.targetContext = ls
end

function save()
	game:saveGame()
end

function title(title)
	game:findQueue({
		id = "title",
		context = "game",
		type = "instant",
		title = title,
		run = function(self)
			game.title = title
		end
	})
end

function code(func, options)
	game:findQueue({
		context = "any",
		type = "instant",
		id = "custom_code",
		options = options,
		run = func
	})
end

function queue_script(s)
	game:findQueue({
		context = "any",
		type = "instant",
		script = s,
		id = "queued_script",
		run = function(self)
			script(self.script)
		end
	})
end

function add(entity, options)
	game:findQueue({
		context = "any",
		type = "instant",
		entity = entity,
		options = options,
		run = function(self)
			game:add(self.entity, self.options)
		end
	})
end

function get(...)
	return game:get(...)
end

function remove(...)
	return game:get(...)
end

function reset()
	game:findQueue({
		context = "scene",
		type = "instant",
		id = "reset",
		run = function(self)
			game.scenes[game.activeScene] = require(string.format('game.scenes.%s', game.activeScene))()
			load_room(game.room)
		end
	})
end

function wait(length)
	game:findQueue({
		context = "any",
		type = "async",
		length = length,
		id = "wait",
		start = function(self)
			self.waited = 0
		end,
		update = function(self, dt, speed)
			self.waited = self.waited + (dt * speed)

			if self.waited > self.length then
				self.finished = true
			end
		end
	})
end

function follow(e)
	game:findQueue({
		id = "follow",
		entity = e,
		context = "game",
		type = "instant",
		run = function(self)
			game.camera:follow(game:get(self.entity))
		end
	})
end

function music(track)
	game:findQueue({
		id = "music",
		context = "any",
		type = "instant",
		track = track,
		run = function(self)
			if self.track ~= game.playing then
				game.audio:stop("music")

				if self.track ~= nil then
					game.audio:playLooping(self.track, "music")
					game.playing = self.track
				end
			end
		end
	})
end

function fade_out(tag, time)
	game:findQueue({
		id = "fade_out",
		context = "any",
		type = "async",
		t = time,
		tag = tag,
		age = 0,
		update = function(self, dt, speed)
			self.age = self.age + (dt * speed)

			game.audio:volume(self.tag, math.lerp(game.save.settings.volume[self.tag], 0, self.age / self.t))

			if self.age > self.t then
				self.finished = true
			end
		end,
		finish = function(self)
			game.audio:volume(self.tag, 0)
		end
	})
end
function fade_in(tag, time)
	game:findQueue({
		id = "fade_out",
		context = "any",
		type = "async",
		t = time,
		tag = tag,
		age = 0,
		update = function(self, dt, speed)
			self.age = self.age + (dt * speed)

			game.audio:volume(self.tag, math.lerp(0, game.save.settings.volume[self.tag], self.age / self.t))

			if self.age > self.t then
				self.finished = true
			end
		end,
		finish = function(self)
			game.audio:volume(self.tag, game.save.settings.volume[self.tag])
		end
	})
end
function volume(t, vol)
	game:findQueue({
		id = "volume",
		context = "any",
		type = "instant",
		t = t,
		vol = vol,
		age = 0,
		run = function(self, dt, speed)
					game.audio:volume(self.t, self.vol)
		end
	})
end

function sfx(s)
	game:findQueue({
		id = "sfx",
		context = "any",
		type = "instant",
		sound = s,
		run = function(self)
			if self.sound ~= nil then
				game.audio:play(self.sound, "sfx")
			end
		end
	})
end

function fullscreen()
	game.save.settings.fullscreen = not game.save.settings.fullscreen
	local width, height = love.window.getDesktopDimensions()

	if game.save.settings.fullscreen then
		local wScale = width / game.width
		local hScale = height / game.height

		if hScale > wScale then
			game.scale = wScale
		else
			game.scale = hScale
		end
	else
		game.scale = game.save.settings.scale
	end

	love.window.setFullscreen(game.save.settings.fullscreen, "desktop")
end

function screenshot()
	local data = love.graphics.newScreenshot()
	data:encode("png", string.format("%s_%i.png", game.identity, os.time()))
end

function resolution(width, height)
	game:findQueue({
		id = "resolution",
		context = "game",
		type = "instant",
		width = width,
		height = height,
		run = function(self)
			game.width = self.width
			game.height = self.height

			love.window.setMode(game.width * game.scale, game.height * game.scale, {vsync = true})

			for index, layer in pairs(game.layerOrder) do
				game.layers[layer] = love.graphics.newCanvas(game.width, game.height)
				game.layers[layer]:setFilter("nearest", "nearest")
			end

			game.buffer = love.graphics.newCanvas(game.scale * game.width, game.scale * game.height)
			game:getSystem("tilemap").rerenderAll = true
		end
	})
end

function play_animation(anim)
	game:findQueue({
		animation = anim,
		id = "play_animation",
		context = "entity",
		type = "async",
		start = function(self)
			self.parent.animation:setAnimation(self.animation)
			self.parent.animation.loop = false
		end,
		update = function(self, dt, speed)
			self.finished = self.parent.animation.finished
		end
	})
end

function scene(scene, skipQueue)
	if skipQueue then
		if game.scenes[scene] then
			game.queuedScene = scene
		else
			if love.filesystem.exists(string.format("game/scenes/%s.lua", scene)) then
				game.scenes[scene] = require(string.format("game.scenes.%s", scene))()
			else
				game.scenes[scene] = require("game.scenes.main")()
			end

			game.queuedScene = scene
		end
	else
		game:findQueue({
			id = "scene",
			context = "any",
			type = "instant",
			scene = scene,
			run = function(self)
				if game.scenes[self.scene] then
					game.queuedScene = self.scene
				else
					if love.filesystem.exists(string.format("game/scenes/%s.lua", self.scene)) then
						game.scenes[self.scene] = require(string.format("game.scenes.%s", self.scene))()
					else
						game.scenes[self.scene] = require("game.scenes.main")()
					end

					game.queuedScene = self.scene
				end
			end
		})
	end
end

--
-- Runs another script.
-- ``run_script "name"``
-- The "run script" event isn't added to the queue since events need to be queued in order.
--
function script(name)
	if game.scripts[name] then
		game.scripts[name]()
	else
		if love.filesystem.exists(string.format("assets/scripts/%s.lua", name)) then
			game.scripts[name] = love.filesystem.load(string.format("assets/scripts/%s.lua", name))
			printf("Script %s loaded.", name)
			game.scripts[name]()
		else
			error(string.format("File not found - assets/scripts/%s.lua", name))
		end
	end
end

function scriptExists(name)
	return love.filesystem.exists(string.format("assets/scripts/%s.lua", name))
end

function speed(s)
	if s == nil then
		s = 1
	end

	if type(s) ~= "number" then
		error("Attempted to set game speed to something other than a number.")
	end

	game:findQueue({
		id = "speed",
		context = "game",
		type = "instant",
		speed = s,
		run = function(self)
			game.speed = self.speed
		end
	})
end

function set_flag(key, value)
	game:findQueue({
		id = "set_flag",
		context = "any",
		type = "instant",
		key = key,
		value = value,
		run = function(self)
			game.save.flags[self.key] = self.value
		end
	})
end

function get_flag(key)
	return game.save.flags[key]
end

--
-- Lock input.
-- ``lock()``
--
function lock()
	game:findQueue({
		id = "lock",
		context = "any",
		type = "instant",
		run = function(promise, dt, speed)
			game.locked = true
		end
	})
end

--
-- Unlock input.
-- ``unlock()``
--
function unlock()
	game:findQueue({
		id = "unlock",
		context = "any",
		type = "instant",
		run = function(promise, dt, speed)
			game.locked = false
		end
	})
end

function velocity(t)
	game:findQueue({
		id = "velocity",
		context = "entity",
		type = "instant",
		x = t.x,
		y = t.y,
		run = function(self)
			local entity = self.parent

			if self.x ~= nil then
				entity.physics.velocity.x = self.x
			end

			if self.y ~= nil then
				entity.physics.velocity.y = self.y
			end
		end
	})
end

function gravity(t)
	game:findQueue({
		id = "gravity",
		context = "any",
		type = "instant",
		x = t.x,
		y = t.y,
		run = function(self)
			if self.parent then
				self.parent.physics.gravity.x = self.x
				self.parent.physics.gravity.y = self.y
			else
				game:scene().systems.physics.gravity.x = self.x
				game:scene().systems.physics.gravity.y = self.y
			end
		end
	})
end

function drag(t)
	game:findQueue({
		id = "drag",
		context = "entity",
		type = "instant",
		x = t.x,
		y = t.y,
		run = function(self)
			self.parent.physics.drag.x = self.x
			self.parent.physics.drag.y = self.y
		end
	})
end

function qprint(msg)
	game:findQueue({
		id = "qprint",
		context = "any",
		type = "instant",
		msg = msg,
		run = function(self)
			print(self.msg)
		end
	})
end

function areQueuesEmpty(queues)
	local events = 0

	events = events + #game.queue.queue
	events = events + #game:scene().queue.queue

	for index, queue in pairs(queues) do
		if type(queue) == "string" then
			if game:get(queue) then
				events = events + #game:get(queue).queue.queue
			end
		else
			events = events + #queue.queue.queue
		end
	end

	return (events == 0), events
end

function buttonPress(button, call, options)
	game:findQueue{
		id = "button_press",
		context = "any",
		type = "async",
		options = options,
		button = button,
		call = call,
		start = function(self)

		end,
		update = function(self, dt, speed)
			if game.controls:pressed(self.button) then
				print "button pressed"
				self.finished = true
			end
		end,
		finish = function(self)
			if self.call then
				self.call(self.options)
			end
		end
	}
end

--
--
--
--
function tween_to(options)
	if (options.x or options.y) and options.t then
		game:findQueue({
			id = "tween",
			context = "entity",
			type = "async",
			x = options.x,
			y = options.y,
			speed = options.speed or false,
			life = options.t or 1,
			age = 0,
			func = options.func or 'linear',
			start = function(self)
				local entity = self.parent
				self.tween = game.tween.new(self.life, entity, {x = self.x, y = self.y}, self.func)
			end,
			update = function(self, dt, speed)
				self.age = self.age + (dt * (self.speed or speed))
				local done = self.tween:set(self.age)

				self.finished = (self.age > self.life)
			end,
			finish = function(self)
				self.tween = false
			end
		})
	else
		error("Tween called with improper params.")
	end
end

function set_animation(anim, target)
	game:findQueue({
		id = "set_animation",
		context = "entity",
		type = "instant",
		target = target,
		animation = anim,
		run = function(self)
			if self.target then
				self.parent = game:get(self.target)
			end

			if self.parent.animation ~= nil then
				self.parent.animation:setAnimation(self.animation)
			else
				printf("!! Warning: attempted to set animation on an entity with no animation component. (id: %s)", self.parent.name)
			end
		end
	})
end
