--
--
--
--
--

local camera = {
	TWEEN = 1,
	SET = 2,

	PLAYER = 1,
	MENU = 2,

	lockToRoom = true,

	x = 0,
	y = 0,
	z = 1,

	world = {
		top = 0,
		left = 0,
		right = 640,
		bottom = 360
	},

	start = {
		x = 0,
		y = 0
	},
	target = {
		x = 0,
		y = 0
	},
	speed = {
		passive = 2,
		active = 5,
	},
	centerCam = false,
	mode = 2,
	followType = false,
	tweenTime = 0,

	offset = {
		x = 0,
		y = 0
	},
	shakeOffset = {
		x = 0,
		y = 0
	},
	shaking = false,
	length = 0,
	intensity = 0,

	priority = {

	},

	following = false,
}

function camera:shake(length, intensity)
	self.shaking = true

	self.intensity = intensity and intensity or 5
	self.length = length and length or 0.2
end

function camera:getPriority()
	if not self.priority[game.activeScene] then
		self.priority[game.activeScene] = {}
	end

	return self.priority[game.activeScene]
end

function camera:setPriority(entity)
	if not self.priority[game.activeScene] then
		self.priority[game.activeScene] = {}
	end

	if type(entity) == "string" then
		entity = game:get(entity)
	end

	if entity == false then
		return false
	end

	local valid = true

	for index, e in pairs(self.priority[game.activeScene]) do
		if e == entity then
			valid = false
		end
	end

	if not entity.x or not entity.y then
		valid = false
	end

	if valid then
		table.insert(self.priority[game.activeScene], entity)
	else
		print("Entity not valid: %s", entity.name)
		return false
	end
end

function camera:update(dt, speed)
	local horizontal, vertical = 0, 0

	self.length = self.length - (dt * speed)
	if self.length > 0 then
		self.offset.x = math.random(-self.intensity, self.intensity)
		self.offset.y = math.random(-self.intensity, self.intensity)
	else
		self.offset.x = 0
		self.offset.y = 0
	end

	local following = game:get(self.following)

	if self.followType == self.PLAYER then
		self.lockToRoom = true
		self.speed.passive = 2
		self.mode = self.SET
	elseif self.followType == self.MENU then
		self.lockToRoom = false
		self.mode = self.TWEEN
		self.speed.passive = 10
	end

	if horizontal ~= 0 or vertical ~= 0 then
		self.target.x = self.target.x - (horizontal * self.speed.active)
		self.target.y = self.target.y - (vertical * self.speed.active)
	elseif following then
		self.centerCam = (following.class ~= "anchor")
		self:setPosition(following.x, following.y)
	end

	if self.mode == self.TWEEN then
		self.tweenTime = self.tweenTime + (dt * speed * self.speed.passive)
		self.x = math.lerp(self.start.x, self.target.x, self.tweenTime)
		self.y = math.lerp(self.start.y, self.target.y, self.tweenTime)

		if self.tweenTime > 1 then
			self.x = self.target.x
			self.y = self.target.y

			self.target.x = self.x
			self.target.y = self.y
		end
	elseif self.mode == self.SET then
		self.x = self.target.x
		self.y = self.target.y
	end

	if self.lockToRoom then
		self.x = math.clamp(self.x, 0, game.width-game:scene().roomWidth)
		self.y = math.clamp(self.y, 0, game.height-game:scene().roomHeight)
		--[[
		if math.abs(self.x) + game.width > game:scene().roomWidth then
			self.x = game.width
		elseif math.abs(self.x) < game.width then
			self.x = 0
		end

		if math.abs(self.y) + game.height > game:scene().roomHeight then
			self.y = game:scene().roomHeight
		elseif math.abs(self.y) < game.height then
			self.y = 0
		end]]
	end
end

function camera:follow(entity)
	if type(entity) == "table" then
		entity = entity.name
	end

	printf("Now following: %s", entity)

	self.following = entity
end

function camera:getScreenPosition(x, y)
	return x - self.x, y - self.y
end

function camera:unfollow()
	self.following = false
end

function camera:setPosition(x, y, mode, speed)
	if speed then
		self.speed = speed
	end

	if mode then
		self.mode = mode
	end

	self.start.x = self.x
	self.start.y = self.y

	if self.centerCam then
		self.target.x = -x + math.floor(game.width / 2)
		self.target.y = -y + math.floor(game.height / 2)
	else
		self.target.x = -x
		self.target.y = -y
	end

	self.tweenTime = 0
end

return camera
