--
--
--
--
--

return function(self)
	local Map = {
		loaded = false,
		name = "nah",

		width = 30,
		height = 17,
		infinite = false,

		layerHeight = 0,

		tileWidth = 8,
		tileHeight = 8,

		pixelHeight = 17 * 8,
		pixelWidth = 30 * 8,

		tilesets = {},
		hitboxes = {},
		layers = {},
		objects = {},
		properties = {},
		loaded = false
	}

	function Map:init(room)
		if room then
			self:load(room)
		end
	end

	function Map:load(room)
		printf("Attempting to load room: %s", room)
		self.name = room

		if not love.filesystem.exists(f("assets/rooms/%s.json", room)) then
			error(f("Attempted to load room that does not exist: %s", room))
		else
			printf("Room %s exists - beginning loading.", room)
		end

		local room = json(f("assets/rooms/%s.json", room))

		self.height = room.height
		self.width = room.width
		self.infinite = room.infinite

		self.tileHeight = room.tileheight
		self.tileWidth = room.tilewidth

		self.pixelHeight = self.height * self.tileHeight
		self.pixelWidth = self.width * self.tileWidth

		--
		-- Load tilesets
		--
		if room.properties then
			for index, property in pairs(room.properties) do
				self.properties[property.name] = property.value
			end
		end

		for index, tileset in pairs(room.tilesets) do
			self.tilesets[tileset.firstgid] = self:loadTileset(tileset)
		end

		for index, layer in pairs(room.layers) do
			self.layers[layer.id] = self:loadLayer(layer)
		end

		self.loaded = true
	end

	----
	--
	-- Tileset loading
	--
	----
	function Map:loadTileset(tileset)
		if tileset.source then
			local path, filename, ext = GetFileName(tileset.source)
			local sourcePath = "assets/tiles/" .. filename
			printf("Loading tileset source: %s", sourcePath)
			tileset = json(sourcePath)
		end

		local tiles = false

		if tileset.version then
			tiles = self:loadTileset1_2(tileset)
		else
			tiles = self:loadTileset1_0(tileset)
		end

		return tiles
	end

	function Map:loadTileset1_0(tileset)
		printf("Tileset %s is of version 1, loading thusly", tileset.name)

		local path, file, ext = GetFileName(tileset.image)
		local tiles = {
			name = tileset.name,

			image = tileset.image,
			sheet = love.graphics.newImage("assets/tiles/" .. file),
			quads = {},
			tiles = {},

			offset = tileset.firstgid,
			columns = tileset.columns,
			count = tileset.tilecount,
			tileHeight = tileset.tileheight,
			tileWidth = tileset.tilewidth,
			imageHeight = tileset.imageheight,
			imageWidth = tileset.imagewidth,
		}

		local tileCount = 0
		for y = 1, math.floor(tiles.imageHeight / tiles.tileHeight) do
			for x = 1, math.floor(tiles.imageWidth / tiles.tileWidth) do
				tiles.quads[tileCount] = love.graphics.newQuad(
					(x - 1) * tiles.tileWidth,
					(y - 1) * tiles.tileHeight,
					tiles.tileWidth,
					tiles.tileHeight,
					tiles.imageWidth,
					tiles.imageHeight
				)
				tileCount = tileCount + 1
			end
		end

		if tileset.tileproperties then
			for index, tile in pairs(tileset.tileproperties) do
				local tileID = tonumber(index)
				local tileData = {
					index = tileID,
					properties = {}
				}

				for index, property in pairs(tile) do
					tileData.properties[index] = property
				end

				tiles.tiles[tileID] = tileData
			end
		end

		if tileset.tiles then
			for index, tile in pairs(tileset.tiles) do
				local tileData = {
					index = tonumber(index),
					properties = {},
					type = tile.type
				}

				if tile.properties then
					for index, property in pairs(tile.properties) do
						tileData.properties[property.name] = property.value
					end
				end

				if tile.animation then
					tileData.frames = {}
					local firstFrame = true
					for index, frame in pairs(tile.animation) do
						table.insert(tileData.frames, { quad = frame.tileid - 1, life = (frame.duration / 1000) })
					end
				end

				tiles.tiles[tonumber(index)] = table.merge(tiles.tiles[tonumber(index)] or {}, tileData)
			end
		end

		return tiles
	end

	function Map:loadTileset1_2(tileset)
		printf("Tileset %s is of version 1.2, loading thusly", tileset.name)

		local path, file, ext = GetFileName(tileset.image)
		local tiles = {
			name = tileset.name,

			image = tileset.image,
			sheet = love.graphics.newImage("assets/tiles/" .. file),
			quads = {},
			tiles = {},

			offset = tileset.firstgid,
			columns = tileset.columns,
			count = tileset.tilecount,
			tileHeight = tileset.tileheight,
			tileWidth = tileset.tilewidth,
			imageHeight = tileset.imageheight,
			imageWidth = tileset.imagewidth,
		}

		--
		--
		-- Building quads.
		--
		--
		local tileCount = 0
		for y = 1, math.floor(tiles.imageHeight / tiles.tileHeight) do
			for x = 1, math.floor(tiles.imageWidth / tiles.tileWidth) do
				tiles.quads[tileCount] = love.graphics.newQuad(
					(x - 1) * tiles.tileWidth,
					(y - 1) * tiles.tileHeight,
					tiles.tileWidth,
					tiles.tileHeight,
					tiles.imageWidth,
					tiles.imageHeight
				)
				tileCount = tileCount + 1
			end
		end

		--
		--
		-- Loading tiles.
		--
		--
		if tileset.tiles then
			for index, tile in pairs(tileset.tiles) do
				local tileData = {
					index = tile.id,
					type = tile.type,
					properties = {}
				}

				if tile.properties then
					for index, property in pairs(tile.properties) do
						tileData.properties[property.name] = property.value
					end
				end

				if tile.animation then
					tileData.frames = {}
					local firstFrame = true
					for index, frame in pairs(tile.animation) do
						table.insert(tileData.frames, { quad = frame.tileid - 1, life = (frame.duration / 1000) })
					end
				end

				if tile.objectgroup then
					tileData.objectgroup = tile.objectgroup
				end

				tiles.tiles[tile.id] = tileData
			end
		end

		return tiles
	end

	----
	--
	-- Layer processing.
	--
	----

	function Map:loadLayer(layer)
		if layer.type == "tilelayer" then
			return self:loadTileLayer(layer)
		elseif layer.type == "objectgroup" then
			return self:loadObjectLayer(layer)
		elseif layer.type == "imagelayer" then
			return self:loadImageLayer(layer)
		elseif layer.type == "group" then
			return self:loadGroupLayer(layer)
		end
	end

	function Map:loadObjectLayer(layer)
		local layer = {
			type = "objects",
			objects = layer.objects,
			properties = layer.properties or {},
			name = layer.name
		}

		local entities = {}

		for index, object in pairs(layer.objects) do
			local name = object.name

			if not name or name == "" then
				name = nil
			end

			local entity = {
				x = object.x,
				y = object.y,
				width = object.width,
				height = object.height,
				name = name
			}

			for index, property in pairs(layer.properties) do
				local parts = property.name:separate('.')
				entity = table.recurse(entity, parts, property.value)
			end

			if object.properties then
				for index, property in pairs(object.properties) do
					local parts = property.name:separate('.')
					entity = table.recurse(entity, parts, property.value)
				end
			end

			if object.type ~= "" then
				table.insert(entities, game:template(object.type, entity))
			else
				table.insert(entities, game:template(entity))
			end
		end

		layer.objects = entities

		return layer
	end

	function Map:loadImageLayer(layer)
		local layer = {
			type = "image",
			image = layer.image
		}

		for index, property in pairs(layer.properties) do
			local parts = property.name:separate('.')
			layer = table.recurse(layer, parts, property.value)
		end

		return layer
	end

	function Map:loadGroupLayer(layer)
		local layer = {
			type = "group",
			name = layer.name,
			layers = layer.layers,
			children = {}
		}

		for index, l in pairs(layer.layers) do
			table.insert(layer.children, self:loadLayer(l))
		end

		return layer
	end

	function Map:loadTileLayer(layer)
		local layer = {
			type = "tile",
			data = layer.data,
			parent = self,

			x = layer.x,
			y = layer.y,
			width = layer.width,
			height = layer.height,

			grid = create_2d_array(layer.width, layer.height, 0),
			layer = 'map',
			canvas = love.graphics.newCanvas(layer.width * self.tileWidth, layer.height * self.tileHeight),
			opacity = layer.opacity,

			properties = layer.properties,
			visible = layer.visible,
			name = layer.name
		}

		--
		-- Move each tile from data to the layer grid.
		--
		for y = 1, layer.height do
			for x = 1, layer.width do
				local tile = layer.data[(x + (layer.width * (y - 1)))]
				layer.grid[y][x] = tile
			end
		end

		return layer
	end

	--
	--
	-- Scenifying
	--
	--
	function Map:scenify(validate)
		if not self.loaded then
			error "Attempted to scenify unloaded map."
		end

		if self.properties.type ~= "menu" then
			game.room = self.name
		end

		self.layerHeight = 0

		for index, layer in pairs(self.layers) do
			if validate then
				if validate(layer) then
					self:scenifyLayer(layer)
				end
			else
				self:scenifyLayer(layer)
			end
		end

		if self.properties.music then
			music(self.properties.music)
		end

		if self.properties.script and game.mapMode == "room" then
			script(self.properties.script)
		end

		return self
	end

	function Map:scenifyLayer(layer)
		printf("Loading %s layer - %s", layer.type, layer.name)

		if layer.type == "tile" then
			self:scenifyTileLayer(layer)
		elseif layer.type == "objects" then
			self:scenifyObjectLayer(layer)
		elseif layer.type == "group" then
			self:scenifyGroupLayer(layer)
		elseif layer.type == "image" then
			self:scenifyImageLayer(layer)
		end

		self.layerHeight = self.layerHeight + 1
	end

	function Map:scenifyImageLayer(layer)
		sprite(layer.source)
	end

	function Map:scenifyGroupLayer(layer)
		for index, child in pairs(layer.children) do
			self:scenifyLayer(child)
		end
	end

	function Map:scenifyTileLayer(layer)
		if #self.tilesets == 0 then
			return false
		end

		local entity = {
			x = layer.x,
			y = layer.y,
			z = self.layerHeight,
			name = layer.name,
			class = "map",
			map = {
				height = layer.height,
				width = layer.width,
				parent = self,
				version = 1.2,
				canvas = layer.canvas,
				hitboxes = {},
				opacity = layer.opacity,
				tiles = layer.grid,
				animatedTiles = {}
			},
			parent = self,
			getSheet = function(self, id)
				local tileset

				for first, t in pairs(self.parent.tilesets) do
					if id >= first then
						offset = t.offset
						tileset = t
					end
				end

				return tileset
			end
		}

		if layer.properties then
			for index, property in pairs(layer.properties) do
				local parts = property.name:separate('.')
				entity = table.recurse(entity, parts, property.value)
			end
		end

		local lc = love.graphics.getCanvas()
		love.graphics.setCanvas(layer.canvas)

		for y = 1, layer.height do
			for x = 1, layer.width do
				local tile = layer.grid[y][x]
				local tileset = self.tilesets[1]
				local offset = 1

				for first, t in pairs(self.tilesets) do
					if tile >= first then
						offset = first
						tileset = t
					end
				end

				tile = tile - offset

				if tile >= 0 then
					if tileset.quads[tile] then
						love.graphics.draw(tileset.sheet, tileset.quads[tile], (x - 1) * tileset.tileWidth, (y - 1) * tileset.tileHeight)
					else
						printf("Tile quad not found: %s:%i", tileset.name, tile)
					end
				end

				if tileset.tiles[tile] then
					if tileset.tiles[tile].properties.collides
					or tileset.tiles[tile].objectgroup then
						if tileset.tiles[tile].objectgroup then
							local hitboxes = {}

							for index, object in pairs(tileset.tiles[tile].objectgroup.objects) do
								local hitbox = {
									parent = entity,
									type = "map",
									class = tileset.tiles[tile].type or "tile",
									collides = tileset.tiles[tile].properties.collides,
									id = tile,
									properties = tileset.tiles[tile].properties
								}
								table.insert(hitboxes, hitbox)
								game:getSystem("physics").world:add(hitbox, (x - 1) * tileset.tileWidth + object.x, (y - 1) * tileset.tileHeight + object.y, object.width, object.height)
							end

							entity.map.hitboxes[f("%i,%i", x, y)] = hitboxes
						else
							local hitbox = {
								parent = entity,
								type = "map",
								class = tileset.tiles[tile].type or "tile",
								id = tile,
								collides = tileset.tiles[tile].properties.collides,
								properties = tileset.tiles[tile].properties
							}
							entity.map.hitboxes[f("%i,%i", x, y)] = {hitbox}
							game:getSystem("physics").world:add(hitbox, (x - 1) * tileset.tileWidth, (y - 1) * tileset.tileHeight, tileset.tileWidth, tileset.tileHeight)
						end
					end

					if tileset.tiles[tile].frames then
						local age = 0

						if tileset.tiles[tile].properties.animation_offset then
							age = (love.math.random() * tileset.tiles[tile].properties.animation_offset) / 100
						end

						entity.map.animatedTiles[f("%i,%i", x, y)] = {
							animation = tileset.tiles[tile].frames,
							x = x,
							tileset = tileset,
							y = y,
							age = age,
							index = 1,
							id = tile
						}

					end

				end
			end
		end

		love.graphics.setCanvas(lc)

		game:add(entity)
	end

	function Map:scenifyObjectLayer(layer)
		for index, object in pairs(layer.objects) do
			game:add(object)
		end
	end

	return Map
end
