
return function(r)
	local map = json(string.format("assets/rooms/%s.json", r))
	local tileAnimations = {}
	local zIndex = 0

	if map.tilesets[1] then
		if map.tilesets[1].source then
			local path, filename, ext = GetFileName(map.tilesets[1].source)
			map.tilesets[1] = json("assets/tiles/" .. reduce_path(filename))
			map.tilesets[1].image = "tiles/" .. map.tilesets[1].image
		end

		if map.tilesets[1].tiles then
			print "Loading tileset animations"
			for index, tile in pairs(map.tilesets[1].tiles) do
				tileAnimations[tonumber(index)] = {}

				if tile.animation then
					for indexb, frame in pairs(tile.animation) do
						table.insert(tileAnimations[tonumber(index)], {quad = frame.tileid, life = frame.duration / 1000})
					end
				end

				print(table.inspect(tileAnimations))
			end
		end
	end

	for index, layer in ipairs(map.layers) do
		if (layer.type == "objectgroup") then
			printf("Loading object layer %s from room %s", layer.name, r)
			for index, object in pairs(layer.objects) do
				local properties = {}

				if layer.properties then
					for key, value in pairs(layer.properties) do
						local parts = key:separate('.')
						properties = table.recurse(properties, parts, value)
					end
				end

				if object.properties then
					for key, value in pairs(object.properties) do
						local parts = key:separate('.')
						properties = table.recurse(properties, parts, value)
					end
				end

				properties.name = object.name
				properties.x = object.x
				properties.y = object.y
				properties.width = object.width
				properties.height = object.height

				game:add(object.type and object.type or {}, properties)
			end
		end

		if (layer.type == "tilelayer") and #map.tilesets == 1 then
			printf("Loading tile layer %s from room %s", layer.name, r)

			zIndex = zIndex + 1

			local entity = {
				x = layer.x,
				y = layer.y,
				z = zIndex,
				name = layer.name,
				class = "map",
				map = {
					version = 1,
					width = layer.width,
					height = layer.height,

					tile = {
						width = map.tilewidth,
						height = map.tileheight
					},

					tiles = {},
					animations = tileAnimations,
					animatedTiles = {},
					layer = "map"
				},
				physics = {
					type = "tilemap"
				}
			}

			if layer.properties then
				for key, value in pairs(layer.properties) do
					local parts = key:separate('.')
					entity = table.recurse(entity, parts, value)
				end
			end

			local tileset

			if #map.tilesets > 1 then
				error "Too many tilesets presented. Keep it to one, dude."
			else
				tileset = map.tilesets[1]

				if tileset.source then
					print(tileset.source)
					print(reduce_path(tileset.source))
					tileset = game.json.decode(love.filesystem.read("assets/" .. reduce_path(tileset.source)))
				end
			end


			for y = 1, layer.height do
				table.insert(entity.map.tiles, {})

				for x = 1, layer.width do
					local tile = layer.data[(x + (layer.width * (y - 1)))]

					table.insert(entity.map.tiles[y], tile)

					if tileAnimations[tile - 1] then
						table.insert(entity.map.animatedTiles,
							{
								x = x, y = y,
								animation = tileAnimations[tile - 1],
								index = 1,
								age = 0,
							}
						)
					end
				end
			end

			if roomWidth < layer.width then
				roomWidth = layer.width
			end

			if roomHeight < layer.height then
				roomHeight = layer.height
			end

			entity.map.tilesheet = love.graphics.newImage(string.format("assets/%s", reduce_path(tileset.image)))
			entity.map.canvas = love.graphics.newCanvas(layer.width * map.tilewidth, layer.height * map.tileheight)
			entity.map.quads = {}
			entity.map.hitboxes = {}

			for y = 1, math.floor(tileset.tilecount / tileset.columns) do
				for x = 1,tileset.columns do
					table.insert(entity.map.quads, love.graphics.newQuad((x - 1) * map.tilewidth, (y - 1) * map.tileheight, map.tilewidth, map.tileheight, tileset.imagewidth, tileset.imageheight))
				end
			end

			local lc = love.graphics.getCanvas()
			love.graphics.setCanvas(entity.map.canvas)

			for y = 1, layer.height do
				for x = 1, layer.width do
					if entity.map.tiles[y][x] ~= 0 then
						love.graphics.draw(
							entity.map.tilesheet,
							entity.map.quads[entity.map.tiles[y][x]],
							(x - 1) * map.tilewidth,
							(y - 1) * map.tileheight
						)

						if tileset.tileproperties then
							for id, props in pairs(tileset.tileproperties) do
								if (id + 1) == entity.map.tiles[y][x] then
									local hitbox = { parent = entity, type = "map" }
									game:getSystem("physics").world:add(hitbox, (x - 1) * map.tilewidth, (y - 1) * map.tileheight, map.tilewidth, map.tileheight)
									table.insert(entity.map.hitboxes, hitbox)
								end
							end
						end
					end
				end
			end

			love.graphics.setCanvas(lc)
			game:add(entity)
		end
	end

	if map.properties then
		if map.properties.map then
			game.room = r
		end

		if map.properties.script then
			script(map.properties.script)
		end

		if map.properties.background then
			game:add {
				x = 0,
				y = 0,
				sprite = {
					sprite = map.properties.background,
					anchored = true,
					origin = {
						x = 0,
						y = 0
					},
					layer = "bg_effect"
				}
			}
		end

		if map.properties.music then
			game.audio:stop("music")
			game.audio:playLooping(map.properties.music, "music")
		end
	end

	map.pixelWidth = roomWidth * map.tileheight
	map.pixelHeight = roomHeight * map.tilewidth

	return map
end
