--
--
--
--
--

return function()
	local animation = game.system()

	function animation:validate(entity)
		if entity.animation then
			entity.animation.parent = entity
			entity.animation = table.merge(require("game.components.animation")(), entity.animation)

			if entity.animation.schema then
				if love.filesystem.exists(entity.animation.schema) then
					local scheme = game.json.decode(love.filesystem.read(entity.animation.schema))

					entity.animation.frames = {}

					for index, frame in ipairs(scheme.frames) do
						table.insert(entity.animation.frames,
							{
								age = (frame.duration / 1000),
								x = frame.frame.x,
								y = frame.frame.y,
								w = frame.frame.w,
								h = frame.frame.h,
								index = index,
								quad = love.graphics.newQuad(
									frame.frame.x, frame.frame.y,
									frame.frame.w, frame.frame.h,
									scheme.meta.size.w, scheme.meta.size.h
								)
							}
						)

						entity.animation.width = frame.frame.w
						entity.animation.height = frame.frame.h
					end

					entity.animation.animations = {}

					for index, animation in pairs(scheme.meta.frameTags) do
						entity.animation.animations[animation.name] = {
							from = animation.from + 1,
							to = animation.to + 1
						}
					end
				else
					error(string.format("Given schema file doesn't exist: %s", entity.animation.schema))
				end
			end

			if not entity.animation.animations[entity.animation.current] then
				local current = false
				for key, deal in pairs(entity.animation.animations) do
					if not current then
						current = key
					end
				end

				entity.animation.current = current
			end

			entity.animation:setAnimation(entity.animation.current)
		end

		return entity
	end

	function animation:filter(entity)
		return entity.animation
	end

	function animation:process(entity, dt, speed)
		local _speed = 1

		if entity.animation.keepSpeed then
			_speed = speed

			if entity.animation.speed then
				_speed = _speed * entity.animation.speed
			end
		else
			_speed = entity.animation.speed
		end


		entity.animation.age = entity.animation.age + (dt * _speed)

		if entity.animation.age > entity.animation.frames[entity.animation.index].age and not entity.animation.finished then
			while entity.animation.age > entity.animation.frames[entity.animation.index].age do
				entity.animation.age = entity.animation.age - entity.animation.frames[entity.animation.index].age
				entity.animation.index = entity.animation.index + 1

				if entity.animation.index > entity.animation.animations[entity.animation.current].to then
					if entity.animation.loop then
						entity.animation.index = entity.animation.animations[entity.animation.current].from
						entity.animation.loops = entity.animation.loops + 1
					else
						entity.animation.finished = true
						entity.animation.index = entity.animation.animations[entity.animation.current].to
					end
				end
			end
		end

		entity.animation.frame = entity.animation.frames[entity.animation.index]

		if entity.sprite then
			entity.sprite.sliceIndex = entity.animation.index
		end
	end

	return animation
end
