--
--
--
--
--

return function()
	local physics = game.system()

	physics.world = game.physics.newWorld(32)
	physics.gravity = {
		x = 0,
		y = 0
	}

	function physics:add(entity)
		if entity.physics.type == "sprite" then
			if not self.world:hasItem(entity) then
				self.world:add(entity, entity.x, entity.y, entity.physics.width, entity.physics.height)
			end
		end

		entity.physics.parent = entity
	end

	function physics:remove(entity)
		if self.world:hasItem(entity) then
			self.world:remove(entity)
		end
	end

	function physics:validate(entity)
		print(string.format("Registering %s's hitbox to world", entity.name))
		entity.physics = table.merge(require("game.components.physics")(), entity.physics)
		if entity.physics.useTiledWidth then
			entity.physics.width = entity.width
			entity.physics.height = entity.height
		end

		return entity
	end

	function physics:filter(entity)
		return entity.physics
	end

	function physics:process(entity, dt, speed)
		entity.center = {
			x = entity.x + entity.physics.offset.x + (entity.physics.width / 2),
			y = entity.y + entity.physics.offset.y + (entity.physics.height / 2),
		}

		if entity.physics.type == "sprite" then
			if not entity.physics.anchored then
				entity.physics.velocity.x = entity.physics.velocity.x + (entity.physics.gravity.x and entity.physics.gravity.x or self.gravity.x)
				entity.physics.velocity.y = entity.physics.velocity.y + (entity.physics.gravity.y and entity.physics.gravity.y or self.gravity.y)
			end

			entity.physics.velocity.x = entity.physics.velocity.x - (math.sign(entity.physics.velocity.x) * (entity.physics.drag.x))
			entity.physics.velocity.y = entity.physics.velocity.y - (math.sign(entity.physics.velocity.y) * (entity.physics.drag.y))

			if entity.physics.velocity.max.x < math.abs(entity.physics.velocity.x) then
				entity.physics.velocity.x = math.sign(entity.physics.velocity.x) * entity.physics.velocity.max.x
			end

			if entity.physics.velocity.max.y < math.abs(entity.physics.velocity.y) then
				entity.physics.velocity.y = math.sign(entity.physics.velocity.y) * entity.physics.velocity.max.y
			end

			if math.abs(entity.physics.velocity.x) <= entity.physics.drag.x then
				entity.physics.velocity.x = 0
			end

			if math.abs(entity.physics.velocity.y) <= entity.physics.drag.y then
				entity.physics.velocity.y = 0
			end

			local move = {
				x = entity.physics.velocity.x,
				y = entity.physics.velocity.y
			}

			self.world:update(entity,
				entity.x + entity.physics.offset.x,
				entity.y + entity.physics.offset.y,
				entity.physics.width,
				entity.physics.height
			)

			local x, y, cols, len = self.world:move(entity,
				entity.x + (move.x * (dt * speed)) + entity.physics.offset.x,
				entity.y + (move.y * (dt * speed)) + entity.physics.offset.y,
				entity.physics.collide
			)


			local moveX = x ~= (entity.x + entity.physics.offset.x)
			local moveY = y ~= (entity.y + entity.physics.offset.y)

			entity.physics.lastTouch.bottom = entity.physics.touch.bottom
			entity.physics.lastTouch.right = entity.physics.touch.right
			entity.physics.lastTouch.left = entity.physics.touch.left
			entity.physics.lastTouch.top = entity.physics.touch.top

			entity.physics.touch.bottom = false
			entity.physics.touch.right = false
			entity.physics.touch.left = false
			entity.physics.touch.top = false

			if len > 0 then
				for index, collision in pairs(cols) do
					if collision.type ~= "cross" then
						if collision.normal.y == -1 then
							entity.physics.touch.bottom = true
						end
						if collision.normal.y == 1 then
							entity.physics.touch.top = true
						end
						if collision.normal.x == -1 then
							entity.physics.touch.left = true
						end
						if collision.normal.x == 1 then
							entity.physics.touch.right = true
						end

						if collision.normal.x ~= 0 then
							if math.sign(entity.physics.velocity.x) == -collision.normal.x then
								entity.physics.velocity.x = entity.physics.velocity.x * entity.physics.bounce.x
							end
						end
						if collision.normal.y ~= 0 then
							if math.sign(entity.physics.velocity.y) == -collision.normal.y then
								entity.physics.velocity.y = entity.physics.velocity.y * entity.physics.bounce.y
							end
						end
					end

					if entity.behavior then
						entity.behavior:collide(collision)
					end

					if collision.other.behavior then
						collision.other.behavior:collide(collision, true)
					end
				end
			end

			if not entity.physics.lock then
				if moveX then
					entity.x = x - entity.physics.offset.x
				end

				if moveY then
					entity.y = y - entity.physics.offset.y
				end
			end
		end
	end

	function physics:draw()
		if game.debugMode then
			local lc = love.graphics.getCanvas()
			love.graphics.setCanvas(game.layers.debug)
			for index, hitbox in pairs(self.world:getItems()) do
				local x, y, w, h = self.world:getRect(hitbox)
				local cursor = game:get("cursor")

				if not cursor then
					cursor = {
						x = game.mouse.x,
						y = game.mouse.y
					}
				end

				if  hitbox.physics then
					if hitbox.physics.collides then
						love.graphics.setColor(255, 0, 0, 255)
					else
						love.graphics.setColor(0, 255, 0, 255)
					end
				else
					love.graphics.setColor(255, 255, 255, 64)
				end

				love.graphics.rectangle("line", math.floor(x + game.camera.x), math.floor(y + game.camera.y), math.floor(w), math.floor(h))
			end
			love.graphics.setColor(255, 255, 255)
			love.graphics.setCanvas(lc)
		end
	end

	return physics
end
