--
--
--
--
--

return function()
	local sprite = game.system()

	function sprite:validate(entity)
		entity.sprite = table.merge(require("game.components.sprite")(), entity.sprite)

		return entity
	end

	function sprite:add(entity)
		entity.sprite.data = love.graphics.newImage(entity.sprite.sprite)
		entity.sprite.data:setFilter(entity.sprite.filter, entity.sprite.filter)

		local path, file, ext = GetFileName(entity.sprite.sprite)
		local dataFile = path .. file:sub(1, #file - #ext - 1) .. ".json"

		if love.filesystem.exists(dataFile) then
			local data = json(dataFile)

			for index, slice in pairs(data.meta.slices) do
				printf("Loading slice %s", slice.name)
				entity.sprite.slices[slice.name] = {
					frames = {},
					color = slice.color
				}

				local keyIndex = 1
				local key = slice.keys[keyIndex]

				for index, frame in pairs(entity.animation and entity.animation.frames or slice.keys) do
					if entity.animation then
						if keyIndex < #slice.keys then
							if index == slice.keys[keyIndex + 1].frame + 1 then
								keyIndex = keyIndex + 1
							end
						end
					else
						keyIndex = keyIndex + 1
					end

					key = deepcopy(slice.keys[keyIndex])

					local x,y,w,h = frame.quad:getViewport()

					key.quad = love.graphics.newQuad(
						key.bounds.x + x,
						key.bounds.y + y,
						key.bounds.w,
						key.bounds.h,
						entity.sprite.data:getDimensions()
					)

					table.insert(entity.sprite.slices[slice.name].frames, key)


				end
			end
		end

		entity.sprite.width, entity.sprite.height = entity.sprite.data:getDimensions()
	end

	function sprite:filter(entity)
		return entity.sprite
	end

	function sprite:render(entity)
		if entity.sprite.visible then
			local lastCanvas = love.graphics.getCanvas()
			love.graphics.setCanvas(game.layers[entity.sprite.layer])
			local r, g, b, a = love.graphics.getColor()
			if type(entity.sprite.color) == "string" then
				love.graphics.setColor(color(entity.sprite.color))
			else
				love.graphics.setColor(entity.sprite.color.r, entity.sprite.color.g, entity.sprite.color.b, entity.sprite.color.a)
			end

			local flipNormals = {
				x = entity.sprite.flip.x and -1 or 1,
				y = entity.sprite.flip.y and -1 or 1
			}

			for y = 0, entity.sprite.tile.y - 1 do
				for x = 0, entity.sprite.tile.x - 1 do
					if entity.animation and entity.animation.frame then
						if entity.sprite.slice then
							love.graphics.draw(entity.sprite.data,
								entity.sprite.slices[entity.sprite.slice]
									.frames
									[entity.animation.index]
									.quad,
								math.floor(entity.x + (x * entity.sprite.slices[entity.sprite.slice].frames[entity.animation.index].bounds.w) + (entity.sprite.flip.x and entity.sprite.slices[entity.sprite.slice].frames[entity.animation.index].bounds.w or 0) + (entity.sprite.offset.x * flipNormals.x) + (entity.sprite.anchored and 0 or game.camera.x)),
								math.floor(entity.y + (y * entity.sprite.slices[entity.sprite.slice].frames[entity.animation.index].bounds.h) + (entity.sprite.flip.y and entity.sprite.slices[entity.sprite.slice].frames[entity.animation.index].bounds.h or 0) + (entity.sprite.offset.y * flipNormals.y) + (entity.sprite.anchored and 0 or game.camera.y)),
								entity.sprite.angle,
								entity.sprite.scale.x * flipNormals.x,
								entity.sprite.scale.y * flipNormals.y,
								math.floor(entity.sprite.slices[entity.sprite.slice].frames[entity.animation.index].bounds.w * entity.sprite.origin.x),
								math.floor(entity.sprite.slices[entity.sprite.slice].frames[entity.animation.index].bounds.h * entity.sprite.origin.y),
								entity.sprite.shear.x,
								entity.sprite.shear.y
							)
						else
							love.graphics.draw(entity.sprite.data,
								entity.animation.frame.quad,
								math.floor(entity.x + (x * entity.animation.width) + (entity.sprite.flip.x and entity.animation.width or 0) + (entity.sprite.offset.x * flipNormals.x) + (entity.sprite.anchored and 0 or game.camera.x)),
								math.floor(entity.y + (y * entity.animation.height) + (entity.sprite.flip.y and entity.animation.height or 0) + (entity.sprite.offset.y * flipNormals.y) + (entity.sprite.anchored and 0 or game.camera.y)),
								entity.sprite.angle,
								entity.sprite.scale.x * flipNormals.x,
								entity.sprite.scale.y * flipNormals.y,
								math.floor(entity.animation.width * entity.sprite.origin.x) * flipNormals.x,
								math.floor(entity.animation.height * entity.sprite.origin.y) * flipNormals.y,
								entity.sprite.shear.x,
								entity.sprite.shear.y
							)
						end
					else
						if entity.sprite.slice then
							love.graphics.draw(entity.sprite.data,
								entity.sprite.slices[entity.sprite.slice]
									.frames[1]
									.quad,
								math.floor(entity.x + (x * entity.sprite.slices[entity.sprite.slice].frames[entity.animation.index].bounds.w) + (entity.sprite.flip.x and entity.animation.width or 0) + (entity.sprite.offset.x * flipNormals.x) + (entity.sprite.anchored and 0 or game.camera.x)),
								math.floor(entity.y + (y * entity.sprite.slices[entity.sprite.slice].frames[entity.animation.index].bounds.h) + (entity.sprite.flip.y and entity.animation.height or 0) + (entity.sprite.offset.y * flipNormals.y) + (entity.sprite.anchored and 0 or game.camera.y)),
								entity.sprite.angle,
								entity.sprite.scale.x * flipNormals.x,
								entity.sprite.scale.y * flipNormals.y,
								math.floor(entity.sprite.slices[entity.sprite.slice].frames[entity.animation.index].bounds.w * entity.sprite.origin.x),
								math.floor(entity.sprite.slices[entity.sprite.slice].frames[entity.animation.index].bounds.h * entity.sprite.origin.y),
								entity.sprite.shear.x,
								entity.sprite.shear.y
							)
						else
							love.graphics.draw(entity.sprite.data,
								math.floor(entity.x + (x * entity.sprite.width) + (entity.sprite.flip.x and entity.sprite.width or 0) + (entity.sprite.offset.x * flipNormals.x) + (entity.sprite.anchored and 0 or game.camera.x)),
								math.floor(entity.y + (y * entity.sprite.height) + (entity.sprite.flip.y and entity.sprite.height or 0) + (entity.sprite.offset.y * flipNormals.y) + (entity.sprite.anchored and 0 or game.camera.y)),
								entity.sprite.angle,
								entity.sprite.scale.x * flipNormals.x,
								entity.sprite.scale.y * flipNormals.y,
								math.floor(entity.sprite.width * entity.sprite.origin.x),
								math.floor(entity.sprite.height * entity.sprite.origin.y),
								entity.sprite.shear.x,
								entity.sprite.shear.y
							)
						end
					end
				end
			end


			if game.debugMode and false then
				local lc = game:setLayer "debug"

				for index, slice in pairs(entity.sprite.slices) do
					love.graphics.setColor(hex2rgb(slice.color))
					local bounds = slice.frames[entity.animation.index].bounds
					love.graphics.rectangle("line", entity.x + bounds.x + game.camera.x, entity.y + bounds.y + game.camera.y, bounds.w, bounds.h)
				end

				love.graphics.setColor(255, 255, 255, 255)
				love.graphics.setCanvas(lc)
			end

			love.graphics.setColor(r, g, b, a)
			love.graphics.setCanvas(lastCanvas)
		end
	end

	return sprite
end
