--
--
--
--
--

require 'game.functions'
require 'game.api'
require 'assets.api'
table.inspect = require 'lib.inspect'
game = require 'game'

function love.load()
	love.keyboard.setKeyRepeat(true)
	print "Loading game..."
	game._imageCache = {}
	love.graphics.newI = love.graphics.newImage
	love.graphics.newImage = function(url, ...)
		if not game._imageCache[url] then
			game._imageCache[url] = love.graphics.newI(url, ...)
		end

		return game._imageCache[url]
	end

	game.brownoutShader = love.graphics.newShader [[
	        vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
	        {
				return vec4(0, 0, 0, 0);
	        }
	    ]]
	game.brownout = 0

	_print = print
	print = function(...)
		if game.debugMode then
			_print(...)
		end
	end

	game:load()

end

function love.update(dt)
	--game.lovebird:update(dt)
	game:update(dt)

	if game.debugMode then
		script "debug/update"
	end
end

function love.draw()

	local lc = love.graphics.getCanvas()
	love.graphics.setCanvas(game.buffer)
	love.graphics.setColor(unpack(game.background))
	love.graphics.rectangle("fill", 0, 0, game.width * game.scale, game.height * game.scale)

	love.graphics.setColor(255, 255, 255)
	for index, layer in pairs(game.layerOrder) do
		love.graphics.setCanvas(game.layers[layer])
		love.graphics.clear()
	end

	love.graphics.setCanvas(lc)

	game.promise:draw()
	game:draw()


	local lc = game:setLayer "particles"
	love.graphics.draw(game.particles)
	love.graphics.setCanvas(lc)

	love.graphics.setCanvas(game.buffer)
	for index, layer in pairs(game.layerOrder) do
		if layer == "debug" then
			love.graphics.setShader()
		end

		love.graphics.draw(game.layers[layer],
			0, 0, 0, game.scale
		)
	end


	love.graphics.setShader()
	love.graphics.setCanvas()

	if not game.blackout then
		love.graphics.setColor(255, 255, 255, (1 - game.brownout) * 255)
		love.graphics.draw(game.buffer, game.camera.offset.x, game.camera.offset.y)
		love.graphics.setColor(255, 255, 255, 255)
	end

	if game.debugMode then
		script "debug/draw"
	end

	game:cleanup()

	if game.queuedScene then
		game.activeScene = game.queuedScene
		game.queuedScene = false
	end
end

function love.textedited(text, start, length)
    -- for IME input
    game.suit.textedited(text, start, length)
end

function love.textinput(t)
	-- forward text input to SUIT
	game.suit.textinput(t)
end

function love.keypressed(key)
	-- forward keypresses to SUIT
	game.suit.keypressed(key)
end

function love.filedropped(file)
	print("Dropped: %s", file:getFilename())
	game.dropped = file
end
